@@ -10,7 +10,6 @@ STRIP_DEFAULT="${STRIP_DEFAULT:=1}"
 STRIP_GSTREAMER_PLUGINS="${STRIP_GSTREAMER_PLUGINS:=$STRIP_DEFAULT}"
 STRIP_SOURCE="${STRIP_SOURCE:=0}"
 STRIP_OPENGL="${STRIP_OPENGL:=$STRIP_DEFAULT}"
-ZIP_MODULES="${ZIP_MODULES:=1}"
 CLIENT_ONLY="${CLIENT_ONLY:=0}"
 
 DO_TESTS="${DO_TESTS:-1}"
@@ -22,10 +21,8 @@ if [ "${CLIENT_ONLY}" == "1" ]; then
 	BUILD_ARGS="${BUILD_ARGS} --without-server --without-shadow --without-proxy"
 	DO_TESTS="0"
 else
-	if [ ! -d "html5" ]; then
-		echo "html5 client not found"
-		echo " perhaps run: 'git clone https://github.com/Xpra-org/xpra-html5'"
-		echo " then: ln -sf xpra-html5 html5"
+	if [ ! -e "${JHBUILD_PREFIX}/share/xpra/www/" ]; then
+		echo "the xpra html5 client must be installed in ${JHBUILD_PREFIX}/share/xpra/www/" 
 		exit 1
 	fi
 fi
@@ -59,9 +56,22 @@ rm -f xpra/src_info.py xpra/build_info.py
 ${PYTHON} "./fs/bin/add_build_info.py" "src" "build"
 rm -fr build/* dist/*
 ${PYTHON} ./setup.py clean
+NPROC=`sysctl -n hw.logicalcpu`
+echo "found $NPROC logical CPUs"
+BUILD_EXT_LOG=`pwd`/build_ext.log
+echo "./setup.py build_ext ${BUILD_ARGS}" -j $NPROC
+echo " (see ${BUILD_EXT_LOG} for details - this may take a minute or two)"
+${PYTHON} ./setup.py build_ext ${BUILD_ARGS} -j $NPROC >& ${BUILD_EXT_LOG}
+if [ "$?" != "0" ]; then
+	popd
+	echo "ERROR: install failed"
+	echo
+	tail -n 20 ${BUILD_EXT_LOG}
+	exit 1
+fi
 INSTALL_LOG=`pwd`/install.log
 echo "./setup.py install ${BUILD_ARGS}"
-echo " (see ${INSTALL_LOG} for details - this may take a minute or two)"
+echo " (see ${INSTALL_LOG} for details)"
 ${PYTHON} ./setup.py install ${BUILD_ARGS} >& ${INSTALL_LOG}
 if [ "$?" != "0" ]; then
 	popd
@@ -151,6 +161,14 @@ fi
 echo
 echo "*******************************************************************************"
 echo "calling 'gtk-mac-bundler Xpra.bundle' in `pwd`"
+if [ ! -e "JHBUILD_PREFIX}/lib/charset.alias" ]; then
+	#gtk-mac-bundler chokes if this file is missing
+	touch "${JHBUILD_PREFIX}/lib/charset.alias"
+fi
+if [ ! -e "JHBUILD_PREFIX}/share/icons/hicolor/index.theme" ]; then
+	#gtk-mac-bundler chokes if this file is missing
+	touch "${JHBUILD_PREFIX}/share/icons/hicolor/index.theme"
+fi
 #we have to make sure we use python2 here (not ported yet):
 PYTHON=${PYTHON} python2 ~/.local/bin/gtk-mac-bundler Xpra.bundle
 if [ "$?" != "0" ]; then
@@ -165,14 +183,14 @@ pushd ${LIBDIR} || exit 1
 ln -sf python3.${PYTHON_MINOR_VERSION} python
 cd python
 rmdir config
-if [ "${ZIP_MODULES}" != "1" ]; then
-	echo "unzip site-packages"
-	if [ -e "site-packages.zip" ]; then
-		unzip -nq site-packages.zip
-		rm site-packages.zip
-	else
-		unzip -nq ../python3${PYTHON_MINOR_VERSION}.zip
-	fi
+echo "unzip site-packages"
+if [ -e "site-packages.zip" ]; then
+	unzip -nq site-packages.zip
+	rm site-packages.zip
+fi
+if [ -e "../python3${PYTHON_MINOR_VERSION}.zip" ]; then
+	unzip -nq ../python3${PYTHON_MINOR_VERSION}.zip
+	rm ../python3${PYTHON_MINOR_VERSION}.zip
 fi
 popd
 
@@ -272,11 +290,9 @@ if [ "$STRIP_OPENGL" == "1" ]; then
 fi
 echo " * remove numpy"
 rm -fr $LIBDIR/python/numpy
-if [ "${ZIP_MODULES}" == "1" ]; then
-	pushd $LIBDIR/python
-	zip --move -ur site-packages.zip OpenGL
-	popd
-fi
+pushd $LIBDIR/python
+zip --move -ur site-packages.zip OpenGL
+popd
 echo " * add gobject-introspection (py2app refuses to do it)"
 rsync -rpl $PYTHON_PACKAGES/gi $LIBDIR/python/
 mkdir $LIBDIR/girepository-1.0
@@ -292,6 +308,11 @@ mv ${RSCDIR}/share/xpra/css ${RSCDIR}/
 #unused py2app scripts:
 rm ${RSCDIR}/__boot__.py ${RSCDIR}/__error__.sh
 
+echo " * docs"
+if [ -d "${JHBUILD_PREFIX}/share/xpra/doc" ]; then
+	mkdir -p ${RSCDIR}/share/doc/xpra
+	rsync -rplogt ${JHBUILD_PREFIX}/share/doc/xpra/* ${RSCDIR}/share/doc/xpra/
+fi
 
 if [ "$STRIP_SOURCE" == "1" ]; then
 	echo "removing py if we have the pyc:"
@@ -305,6 +326,9 @@ if [ "$STRIP_SOURCE" == "1" ]; then
 		fi
 	done
 fi
+#always strip the cython html reports:
+echo "removing html cython report"
+find $LIBDIR/python/xpra/ -name "*.html" -exec rm {} \;
 
 
 #gst bits expect to find dylibs in Frameworks!?
@@ -343,13 +367,6 @@ echo "Add the manual in HTML format (since we cannot install the man page proper
 groff -mandoc -Thtml < ../../fs/share/man/man1/xpra.1 > ${RSCDIR}/share/manual.html
 groff -mandoc -Thtml < ../../fs/share/man/man1/xpra_launcher.1 > ${RSCDIR}/share/launcher-manual.html
 
-if [ "${CLIENT_ONLY}" != "1" ]; then
-	echo "Add the HTML5 client"
-	pushd html5
-	python3 ./setup.py install ../${RSCDIR}/www/
-	popd
-fi
-
 echo
 echo "*******************************************************************************"
 echo "adding version \"$VERSION\" and revision \"$REVISION$REV_MOD\" to Info.plist files"
@@ -371,6 +388,25 @@ find ./image -name ".svn" | xargs rm -fr
 #not sure why these get bundled at all in the first place!
 find ./image -name "*.la" -exec rm -f {} \;
 
+echo "*******************************************************************************"
+echo "Remove extra Pillow plugins"
+pushd $LIBDIR/python/PIL
+RMP=""
+KMP=""
+for file_name in `ls *Image*`; do
+	plugin_name=`echo $file_name | sed 's+\.py.*++g'`
+    echo "$file_name" | egrep "Bmp|Ico|Image.py|ImageChops|ImageCms|ImageChops|ImageColor|ImageDraw|ImageFile|ImageFilter|ImageFont|ImageGrab|ImageMode|ImageOps|ImagePalette|ImagePath|ImageSequence|ImageStat|ImageTransform|Jpeg|Tiff|Png|Ppm|Xpm|WebP" >& /dev/null
+	if [ "$?" == "0" ]; then
+		KMP="${KMP} $plugin_name"
+	else
+		RMP="${RMP} $plugin_name"
+		rm $file_name
+	fi
+done
+echo " removed: ${RMP}"
+echo " kept: ${KMP}"
+popd
+
 echo
 echo "*******************************************************************************"
 echo "De-duplicate dylibs"
