@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2011-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -12,10 +12,10 @@ import unittest
 import tempfile
 import uuid
 import hmac
+from time import monotonic
 
 from xpra.os_util import (
     strtobytes, bytestostr,
-    monotonic_time,
     WIN32, OSX, POSIX,
     get_hex_uuid,
     )
@@ -24,7 +24,7 @@ from xpra.net.digest import get_digests, get_digest_module, gendigest
 
 
 def temp_filename(prefix=""):
-    return os.path.join(os.environ.get("TEMP", "/tmp"), "file-auth-%s-test-%s" % (prefix, monotonic_time()))
+    return os.path.join(tempfile.gettempdir(), "file-auth-%s-test-%s" % (prefix, monotonic()))
 
 
 class TempFileContext:
@@ -57,24 +57,25 @@ class TestAuth(unittest.TestCase):
         assert str(mod)
         return mod
 
-    def _init_auth(self, mod_name, username="foo", **kwargs):
+    def _init_auth(self, mod_name, **kwargs):
         mod = self.a(mod_name)
-        a = self.do_init_auth(mod, username, **kwargs)
+        a = self.do_init_auth(mod, **kwargs)
         assert repr(a)
         return a
 
-    def do_init_auth(self, module, username="foo", **kwargs):
+    def do_init_auth(self, module, **kwargs):
         try:
             c = module.Authenticator
         except AttributeError:
-            raise Exception("module %s does not contain an Authenticator class!")
+            raise Exception("module %s does not contain an Authenticator class!") from None
         #some auth modules require this to function:
         if "connection" not in kwargs:
             kwargs["connection"] = "fake-connection-data"
         #exec auth would fail during rpmbuild without a default command:
         if "command" not in kwargs:
-            kwargs["command"] = "/usr/bin/true"
-        return c(username, **kwargs)
+            kwargs["command"] = "/bin/true"
+        kwargs["username"] = kwargs.get("username", "foo")
+        return c(**kwargs)
 
     def _test_module(self, module):
         a = self._init_auth(module)
@@ -323,7 +324,7 @@ class TestAuth(unittest.TestCase):
             #can't be used!
             return
         #no connection supplied:
-        pc = self._init_auth("peercred", {})
+        pc = self._init_auth("peercred")
         assert not pc.requires_challenge()
         assert not self.capsauth(pc)
         assert pc.get_uid()==-1 and pc.get_gid()==-1
@@ -345,7 +346,7 @@ class TestAuth(unittest.TestCase):
         def wait_for_connection():
             conn, addr = sock.accept()
             s = SocketConnection(conn, sockpath, addr, sockpath, "unix")
-            pc = self._init_auth("peercred", username="foo", connection=s)
+            pc = self._init_auth("peercred", connection=s)
             assert not pc.requires_challenge()
             assert pc.get_uid()==os.getuid()
             verified.append(True)
@@ -380,8 +381,8 @@ class TestAuth(unittest.TestCase):
             a = self._init_auth("exec", **kwargs)
             assert not a.requires_challenge(), "%s should not require a challenge" % a
             assert self.capsauth(a)==success, "%s should have %s using cmd=%s" % (a, ["failed", "succeeded"][success], cmd)
-        exec_cmd("/usr/bin/true", True)
-        exec_cmd("/usr/bin/false", False)
+        exec_cmd("/bin/true", True)
+        exec_cmd("/bin/false", False)
 
 
 def main():
