@@ -24,7 +24,6 @@ from gi.repository import Pango, GLib, Gtk, GdkPixbuf
 
 from xpra.gtk_common.gobject_compat import register_os_signals
 from xpra.scripts.config import read_config, make_defaults_struct, validate_config, save_config
-from xpra.codecs.codec_constants import PREFERRED_ENCODING_ORDER
 from xpra.gtk_common.gtk_util import (
     add_close_accel, scaled_image, color_parse,
     choose_file, imagebutton, get_icon_pixbuf,
@@ -32,18 +31,26 @@ from xpra.gtk_common.gtk_util import (
 from xpra.util import DEFAULT_PORT, csv, repr_ellipsized
 from xpra.os_util import WIN32, OSX, POSIX
 from xpra.make_thread import start_thread
-from xpra.client.gtk_base.menu_helper import make_encodingsmenu
 from xpra.gtk_common.about import about
 from xpra.scripts.main import (
-    connect_to, make_client, is_local,
-    add_ssh_args, parse_ssh_string, add_ssh_proxy_args,
+    connect_to, make_client,
     configure_network, configure_env, configure_logging,
     )
-from xpra.platform import get_username
+from xpra.scripts.parsing import is_local, add_ssh_args, parse_ssh_string, add_ssh_proxy_args
+from xpra.exit_codes import RETRY_EXIT_CODES, EXIT_STR
+from xpra.platform.info import get_username
 from xpra.log import Logger, enable_debug_for
 
 log = Logger("launcher")
 
+MODE_NESTED_SSH = "ssh -> ssh"
+MODE_SSH = "ssh"
+MODE_TCP = "tcp"
+MODE_SSL = "ssl"
+MODE_WS = "ws"
+MODE_WSS = "wss"
+
+
 #what we save in the config file:
 SAVED_FIELDS = [
     "username", "password", "host", "port", "mode", "ssh_port",
@@ -73,7 +80,7 @@ LAUNCHER_DEFAULTS = {
                         "port"              : -1,
                         "username"          : get_username(),
                         "password"          : "",
-                        "mode"              : "tcp",    #tcp,ssh,..
+                        "mode"              : MODE_TCP,    #tcp,ssh,..
                         "autoconnect"       : False,
                         "ssh_port"          : 22,
                         "proxy_host"        : "",
@@ -89,6 +96,7 @@ red     = color_parse("red")
 white   = color_parse("white")
 
 
+
 def get_active_item_index(optionmenu):
     menu = optionmenu.get_popup()
     for i, x in enumerate(menu.get_children()):
@@ -141,19 +149,24 @@ class ApplicationWindow:
         self.is_paramiko = ssh_cmd.startswith("paramiko")
 
     def get_connection_modes(self):
-        modes = ["ssh"]
-        modes.append("ssh -> ssh")
+        modes = [MODE_SSH]
+        modes.append(MODE_NESTED_SSH)
         try:
             import ssl
             assert ssl
-            modes.append("ssl")
+            modes.append(MODE_SSL)
         except ImportError:
             pass
         #assume crypto is available
-        modes.append("tcp + aes")
-        modes.append("tcp")
-        modes.append("ws")
-        modes.append("wss")
+        try:
+            from xpra.net.crypto import MODES
+            for mode in MODES:
+                modes.append("tcp + aes-%s" % mode.lower())
+        except ImportError:
+            pass
+        modes.append(MODE_TCP)
+        modes.append(MODE_WS)
+        modes.append(MODE_WSS)
         return modes
 
     def get_launcher_validation(self):
@@ -381,38 +394,6 @@ class ApplicationWindow:
         self.info.modify_fg(Gtk.StateType.NORMAL, red)
         vbox.pack_start(self.info)
 
-        hbox = Gtk.HBox(False, 0)
-        hbox.set_spacing(20)
-        self.advanced_options_check = Gtk.CheckButton("Advanced Options")
-        self.advanced_options_check.connect("toggled", self.advanced_options_toggled)
-        self.advanced_options_check.set_active(False)
-        al = Gtk.Alignment(xalign=0.5, yalign=0.5, xscale=0.0, yscale=0)
-        al.add(self.advanced_options_check)
-        hbox.pack_start(al)
-        vbox.pack_start(hbox)
-        self.advanced_box = Gtk.VBox()
-        vbox.pack_start(self.advanced_box)
-
-        # Encoding:
-        hbox = Gtk.HBox(False, 20)
-        hbox.set_spacing(20)
-        hbox.pack_start(Gtk.Label("Encoding: "))
-        self.encoding_combo = Gtk.MenuButton()
-        encodings = ["auto"]+[x for x in PREFERRED_ENCODING_ORDER]
-        server_encodings = encodings
-        es = make_encodingsmenu(self.get_current_encoding, self.set_new_encoding, encodings, server_encodings)
-        self.encoding_combo.set_popup(es)
-        hbox.pack_start(self.encoding_combo)
-        self.advanced_box.pack_start(hbox)
-        self.set_new_encoding(self.config.encoding)
-        # Sharing:
-        self.sharing = Gtk.CheckButton("Sharing")
-        self.sharing.set_active(bool(self.config.sharing))
-        self.sharing.set_tooltip_text("allow multiple concurrent users to connect")
-        al = Gtk.Alignment(xalign=0.5, yalign=0.5, xscale=0.0, yscale=0)
-        al.add(self.sharing)
-        self.advanced_box.pack_start(al)
-
         # Buttons:
         hbox = Gtk.HBox(False, 20)
         vbox.pack_start(hbox)
@@ -435,7 +416,6 @@ class ApplicationWindow:
         hbox.pack_start(self.connect_btn)
 
         vbox.show_all()
-        self.advanced_options_toggled()
         self.window.vbox = vbox
         self.window.add(vbox)
 
@@ -452,7 +432,14 @@ class ApplicationWindow:
                 for size in (16, 32, 48):
                     scaled = pixbuf.scale_simple(size, size, GdkPixbuf.InterpType.BILINEAR)
                     Gtk.IconTheme.add_builtin_icon(icon_name, size, scaled)
-                pixbuf = theme.load_icon(icon_name, Gtk.IconSize.BUTTON, Gtk.IconLookupFlags.USE_BUILTIN)
+                try:
+                    reload = theme.load_icon(icon_name, Gtk.IconSize.BUTTON, Gtk.IconLookupFlags.USE_BUILTIN)
+                except Exception:
+                    log("button: failed to load icon after adding to builtins", exc_info=True)
+                    size = 32
+                    pixbuf = pixbuf.scale_simple(size, size, GdkPixbuf.InterpType.BILINEAR)
+                else:
+                    pixbuf = reload
         if pixbuf:
             image = Gtk.Image.new_from_pixbuf(pixbuf)
             button.add(image)
@@ -468,8 +455,8 @@ class ApplicationWindow:
 
     def validate(self, *args):
         mode = self.mode_combo.get_active_text().lower()
-        ssh = mode=="ssh"
-        sshtossh = mode=="ssh -> ssh"
+        ssh = mode==MODE_SSH
+        sshtossh = mode==MODE_NESTED_SSH
         errs = []
         host = self.host_entry.get_text()
         errs.append((self.host_entry, not bool(host), "specify the host"))
@@ -547,8 +534,8 @@ class ApplicationWindow:
 
     def mode_changed(self, *_args):
         mode = self.mode_combo.get_active_text().lower()
-        ssh = mode=="ssh"
-        sshtossh = mode=="ssh -> ssh"
+        ssh = mode==MODE_SSH
+        sshtossh = mode==MODE_NESTED_SSH
         if ssh or sshtossh:
             self.port_entry.set_tooltip_text("Display number (optional)")
             self.port_entry.set_text("")
@@ -616,44 +603,11 @@ class ApplicationWindow:
                 self.check_boxes_hbox.hide()
                 self.proxy_password_hbox.hide()
         self.validate()
-        if mode in ("ssl", "wss") or (mode=="ssh" and not WIN32):
+        if mode in (MODE_SSL, MODE_WSS) or (mode==MODE_SSH and not WIN32):
             self.nostrict_host_check.show()
         else:
             self.nostrict_host_check.hide()
 
-    def get_current_encoding(self):
-        return self.config.encoding
-
-    def set_new_encoding(self, e):
-        self.encoding_combo.set_label(e)
-        self.config.encoding = e
-
-    def get_selected_encoding(self, *_args):
-        if not self.encoding_combo:
-            return ""
-        index = get_active_item_index(self.encoding_combo)
-        return self.encoding_combo.get_popup().index_to_encoding.get(index)
-
-    def encoding_changed(self, *args):
-        encoding = self.get_selected_encoding()
-        uses_quality_option = encoding in ["jpeg", "webp", "h264", "auto"]
-        log("encoding_changed(%s) uses_quality_option(%s)=%s", args, encoding, uses_quality_option)
-        if uses_quality_option:
-            self.quality_combo.show()
-            self.quality_label.show()
-        else:
-            self.quality_combo.hide()
-            self.quality_label.hide()
-
-    def advanced_options_toggled(self, *_args):
-        if not self.advanced_box:
-            return
-        show_opts = self.advanced_options_check.get_active()
-        if show_opts:
-            self.advanced_box.show()
-        else:
-            self.advanced_box.hide()
-
     def reset_errors(self):
         self.set_sensitive(True)
         self.set_info_text("")
@@ -734,10 +688,12 @@ class ApplicationWindow:
 
     def connect_builtin(self):
         #cooked vars used by connect_to
-        params = {"type"    : self.config.mode}
-        self.config.sharing = self.sharing.get_active()
         username = self.config.username
-        if self.config.mode=="ssh" or self.config.mode=="ssh -> ssh":
+        params = {
+            "type"      : self.config.mode,
+            "username"  : username,
+            }
+        if self.config.mode==MODE_SSH or self.config.mode==MODE_NESTED_SSH:
             if self.config.socket_dir:
                 params["socket_dir"] = self.config.socket_dir
             params["remote_xpra"] = self.config.remote_xpra
@@ -775,7 +731,7 @@ class ApplicationWindow:
                 params["username"] = username
             if self.nostrict_host_check.get_active():
                 full_ssh += ["-o", "StrictHostKeyChecking=no"]
-            if params["type"] == "ssh -> ssh":
+            if params["type"] == MODE_NESTED_SSH:
                 params["type"] = "ssh"
                 params["proxy_host"] = self.config.proxy_host
                 params["proxy_port"] = self.config.proxy_port
@@ -794,12 +750,12 @@ class ApplicationWindow:
             params["display"] = ":%s" % self.config.port
             params["display_name"] = "unix-domain:%s" % self.config.port
         else:
-            assert self.config.mode in ("tcp", "ssl", "ws", "wss"), "invalid / unsupported mode %s" % self.config.mode
+            assert self.config.mode in (MODE_TCP, MODE_SSL, MODE_WS, MODE_WSS), "invalid / unsupported mode %s" % self.config.mode
             params["host"] = self.config.host
             params["local"] = is_local(self.config.host)
             params["port"] = int(self.config.port)
             params["display_name"] = "%s:%s:%s" % (self.config.mode, self.config.host, self.config.port)
-            if self.config.mode in ("ssl", "wss") and self.nostrict_host_check.get_active():
+            if self.config.mode in (MODE_SSL, MODE_WSS) and self.nostrict_host_check.get_active():
                 params["strict-host-check"] = False
 
         #print("connect_to(%s)" % params)
@@ -807,22 +763,25 @@ class ApplicationWindow:
         #or the config file may contain a username which is different from the default one
         #which is used for initializing the client during init,
         #so update the client now:
-        def raise_exception(*args):
-            raise Exception(*args)
         configure_env(self.config.env)
         configure_logging(self.config, "attach")
         configure_network(self.config)
+        self.start_client(params)
+
+    def start_client(self, display_desc):
+        def raise_exception(*args):
+            raise Exception(*args)
         self.client = make_client(raise_exception, self.config)
         self.client.show_progress(30, "client configuration")
         self.client.init(self.config)
         self.client.show_progress(40, "loading user interface")
         self.client.init_ui(self.config)
-        self.client.username = username
+        self.client.username = display_desc.get("username")
         def handshake_complete(*_args):
             self.client.show_progress(100, "connection established")
         self.client.after_handshake(handshake_complete)
         self.set_info_text("Connecting...")
-        start_thread(self.do_connect_builtin, "connect", daemon=True, args=(params,))
+        start_thread(self.do_connect_builtin, "connect", daemon=True, args=(display_desc,))
 
     def ssh_failed(self, message):
         log("ssh_failed(%s)", message)
@@ -847,7 +806,6 @@ class ApplicationWindow:
         log("connect_to(..)=%s, hiding launcher window, starting client", conn)
         GLib.idle_add(self.start_XpraClient, conn, display_desc)
 
-
     def start_XpraClient(self, conn, display_desc):
         try:
             self.do_start_XpraClient(conn, display_desc)
@@ -885,13 +843,25 @@ class ApplicationWindow:
                     self.set_sensitive(True)
                     GLib.idle_add(w.show)
 
+        def reconnect(exit_code):
+            log("reconnect(%s) config reconnect=%s",
+                EXIT_STR.get(exit_code, exit_code), self.config.reconnect)
+            if not self.config.reconnect or exit_code not in RETRY_EXIT_CODES:
+                return False
+            self.clean_client()
+            #give time for the main loop to run once after calling cleanup
+            GLib.timeout_add(100, self.start_client, display_desc)
+            return True
+
         def warn_and_quit_override(exit_code, warning):
             log("warn_and_quit_override(%s, %s)", exit_code, warning)
-            if self.exit_code is None:
-                self.exit_code = exit_code
             password_warning = warning.find("invalid password")>=0
             if password_warning:
                 self.password_warning()
+            elif reconnect(exit_code):
+                return
+            if self.exit_code is None:
+                self.exit_code = exit_code
             err = exit_code!=0 or password_warning
             if not self.current_error:
                 self.current_error = warning
@@ -901,6 +871,8 @@ class ApplicationWindow:
 
         def quit_override(exit_code):
             log("quit_override(%s)", exit_code)
+            if reconnect(exit_code):
+                return
             if self.exit_code is None:
                 self.exit_code = exit_code
             handle_client_quit(self.exit_code==0)
@@ -918,8 +890,8 @@ class ApplicationWindow:
             #no need to start a new main loop:
             self.client.gtk_main = noop
         try:
-            self.client.run()
-            log("client.run() returned")
+            r = self.client.run()
+            log("client.run() returned %s", r)
         except Exception as e:
             log.error("client error", exc_info=True)
             self.handle_exception(e)
@@ -976,17 +948,16 @@ class ApplicationWindow:
         if self.is_putty:
             self.config.proxy_key = self.proxy_key_entry.get_text()
 
-        self.config.encoding = self.get_selected_encoding() or self.config.encoding
         mode_enc = self.mode_combo.get_active_text().lower()
-        if mode_enc.startswith("tcp"):
-            self.config.mode = "tcp"
+        if mode_enc.startswith(MODE_TCP):
+            self.config.mode = MODE_TCP
             if mode_enc.find("aes")>0:
-                self.config.encryption = "AES"
-        elif mode_enc in ("ssl", "ssh", "ws", "wss", "ssh -> ssh"):
+                self.config.encryption = "AES-"+mode_enc.split("aes-")[1].upper()
+        elif mode_enc in (MODE_SSL, MODE_SSH, MODE_WS, MODE_WSS, MODE_NESTED_SSH):
             self.config.mode = mode_enc
             self.config.encryption = ""
         log("update_options_from_gui() %s",
-            (self.config.username, self.config.password, self.config.mode, self.config.encryption, self.config.host, self.config.port, self.config.ssh_port, self.config.encoding))
+            (self.config.username, self.config.password, self.config.mode, self.config.encryption, self.config.host, self.config.port, self.config.ssh_port))
 
     def update_gui_from_config(self):
         #mode:
@@ -997,8 +968,6 @@ class ApplicationWindow:
                 active = i
                 break
         self.mode_combo.set_active(active)
-        if self.config.encoding and self.encoding_combo:
-            self.set_new_encoding(self.config.encoding)
         def get_port(vstr, default_port=""):
             try:
                 iport = int(vstr)
@@ -1008,7 +977,7 @@ class ApplicationWindow:
                 pass
             return str(default_port)
         dport = DEFAULT_PORT
-        if mode in ("ssh", "ssh -> ssh"):
+        if mode in (MODE_SSH, MODE_NESTED_SSH):
             #not required, so don't specify one
             dport = ""
         self.port_entry.set_text(get_port(self.config.port, dport))
@@ -1026,7 +995,6 @@ class ApplicationWindow:
         self.proxy_password_entry.set_text(proxy_password)
         if self.is_putty:
             self.proxy_key_entry.set_text(self.config.proxy_key)
-        self.sharing.set_active(bool(self.config.sharing))
         self.username_entry.set_text(username)
         self.password_entry.set_text(password)
         self.host_entry.set_text(self.config.host)
@@ -1049,7 +1017,7 @@ class ApplicationWindow:
         from xpra.scripts.parsing import parse_URL
         address, props = parse_URL(url)
         pa = address.split("://")
-        if pa[0] in ("tcp", "ssh", "ssl", "ws", "wss") and len(pa)>=2:
+        if pa[0] in (MODE_TCP, MODE_SSH, MODE_SSL, MODE_WS, MODE_WSS) and len(pa)>=2:
             props["mode"] = pa[0]
             host = pa[1]
             if host.find("@")>0:
