@@ -1,23 +1,24 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
 import weakref
+from time import monotonic
 from subprocess import Popen, PIPE
 from gi.repository import Gtk, Gdk, GdkPixbuf
 
 from xpra.client.gtk_base.gtk_client_window_base import HAS_X11_BINDINGS, XSHAPE
 from xpra.util import (
-    updict, pver, iround, flatten_dict, noerr,
-    envbool, envint, repr_ellipsized, csv, first_time, typedict,
+    updict, pver, flatten_dict, noerr,
+    envbool, envint, repr_ellipsized, ellipsizer, csv, first_time, typedict,
     DEFAULT_METADATA_SUPPORTED, XPRA_OPENGL_NOTIFICATION_ID,
     )
 from xpra.os_util import (
-    bytestostr, strtobytes, hexstr, monotonic_time, load_binary_file,
+    bytestostr, strtobytes, hexstr, load_binary_file,
     WIN32, OSX, POSIX, is_Wayland,
     )
 from xpra.simple_stats import std_unit
@@ -27,6 +28,7 @@ from xpra.make_thread import start_thread
 from xpra.gtk_common.cursor_names import cursor_types
 from xpra.gtk_common.gtk_util import (
     get_gtk_version_info, scaled_image, get_default_cursor, color_parse,
+    get_icon_pixbuf,
     get_pixbuf_from_data,
     get_default_root_window, get_root_size,
     get_screen_sizes, GDKWindow,
@@ -238,7 +240,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             rates[m] = monitor.get_refresh_rate()
         rate = -1
         if rates:
-            rate = iround(min(rates.values())/1000)
+            rate = round(min(rates.values())/1000)
         if rate<30 or rate>250:
             rate = super().get_vrefresh()
         return rate
@@ -264,7 +266,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
 
 
     def _process_startup_complete(self, packet):
-        UIXpraClient._process_startup_complete(self, packet)
+        super()._process_startup_complete(packet)
         Gdk.notify_startup_complete()
 
 
@@ -272,7 +274,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         authlog = Logger("auth")
         self.show_progress(100, "authentication")
         PINENTRY = os.environ.get("XPRA_PINENTRY", "")
-        from xpra.scripts.main import get_pinentry_command
+        from xpra.scripts.pinentry_wrapper import get_pinentry_command
         pinentry_cmd = get_pinentry_command(PINENTRY)
         authlog("do_process_challenge_prompt%s get_pinentry_command(%s)=%s",
                 (packet, prompt), PINENTRY, pinentry_cmd)
@@ -288,6 +290,16 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             self.pinentry_proc = None
             noerr(pp.terminate)
 
+    def get_server_authentication_string(self):
+        p = self._protocol
+        if p:
+            server_type = {
+                "xpra"  : "Xpra ",
+                "rfb"   : "VNC ",
+                }.get(p.TYPE, p.TYPE)
+        else:
+            server_type = ""
+        return "%sServer Authentication:" % server_type
 
     def handle_challenge_with_pinentry(self, packet, prompt="password", cmd="pinentry"):
         authlog = Logger("auth")
@@ -306,18 +318,20 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                     from xpra.net.bytestreams import pretty_socket
                     cinfo = conn.get_info()
                     endpoint = pretty_socket(cinfo.get("endpoint", conn.target)).split("?")[0]
-                    q += " for %s" % endpoint
+                    q += "\n at %s" % endpoint
             def got_pin(value):
                 self.idle_add(self.send_challenge_reply, packet, value)
             def no_pin():
                 self.idle_add(self.quit, EXIT_PASSWORD_REQUIRED)
-            from xpra.scripts.main import pinentry_getpin
-            pinentry_getpin(proc, "Xpra Server Authentication:", q, got_pin, no_pin)
+            from xpra.scripts.pinentry_wrapper import pinentry_getpin
+            title = self.get_server_authentication_string()
+            pinentry_getpin(proc, title, q, got_pin, no_pin)
             return True
         return self.do_process_challenge_prompt_dialog(packet, prompt)
 
     def do_process_challenge_prompt_dialog(self, packet, prompt="password"):
-        dialog = Gtk.Dialog("Server Authentication",
+        title = self.get_server_authentication_string()
+        dialog = Gtk.Dialog(title,
                None,
                Gtk.DialogFlags.MODAL | Gtk.DialogFlags.DESTROY_WITH_PARENT)
         dialog.add_button(Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT)
@@ -331,7 +345,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         import gi
         gi.require_version("Pango", "1.0")
         from gi.repository import Pango
-        title = Gtk.Label("Server Authentication")
+        title = Gtk.Label(title)
         title.modify_font(Pango.FontDescription("sans 14"))
         add(title, 16)
         add(Gtk.Label(self.get_challenge_prompt(prompt)), 10)
@@ -391,13 +405,8 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             return
         color_str = color_str.replace(":off", "")
         if color_str in ("auto", ""):
-            from hashlib import md5, sha1
-            try:
-                if envbool("XPRA_NOMD5", False):
-                    raise ValueError("md5 explicitly disabled")
-                m = md5()
-            except ValueError:
-                m = sha1()
+            from hashlib import sha256
+            m = sha256()
             endpoint = self.display_desc.get("display_name")
             if endpoint:
                 m.update(strtobytes(endpoint))
@@ -639,13 +648,10 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             force_focus()
             self.session_info.present()
             return
-        pixbuf = self.get_pixbuf("statistics.png")
-        if not pixbuf:
-            pixbuf = self.get_pixbuf("xpra.png")
         p = self._protocol
         conn = p._conn if p else None
         from xpra.client.gtk_base.session_info import SessionInfo
-        self.session_info = SessionInfo(self, self.session_name, pixbuf, conn, self.get_pixbuf)
+        self.session_info = SessionInfo(self, self.session_name, conn)
         self.session_info.set_args(*args)
         force_focus()
         self.session_info.show_all()
@@ -681,23 +687,9 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         self.timeout_add(200, init_bug_report)
 
 
-    def get_pixbuf(self, icon_name):
-        try:
-            if not icon_name:
-                log("get_pixbuf(%s)=None", icon_name)
-                return None
-            icon_filename = get_icon_filename(icon_name)
-            log("get_pixbuf(%s) icon_filename=%s", icon_name, icon_filename)
-            if icon_filename:
-                return GdkPixbuf.Pixbuf.new_from_file(icon_filename)
-        except Exception:
-            log.error("get_pixbuf(%s)", icon_name, exc_info=True)
-        return None
-
-
     def get_image(self, icon_name, size=None):
         try:
-            pixbuf = self.get_pixbuf(icon_name)
+            pixbuf = get_icon_pixbuf(icon_name)
             log("get_image(%s, %s) pixbuf=%s", icon_name, size, pixbuf)
             if not pixbuf:
                 return  None
@@ -792,8 +784,11 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         return get_root_size()
 
 
+    def get_raw_mouse_position(self):
+        return self.get_root_window().get_pointer()[-3:-1]
+
     def get_mouse_position(self):
-        p = self.get_root_window().get_pointer()[-3:-1]
+        p = self.get_raw_mouse_position()
         return self.cp(p[0], p[1])
 
     def get_current_modifiers(self):
@@ -913,7 +908,8 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
     def make_cursor(self, cursor_data):
         #if present, try cursor ny name:
         display = Gdk.Display.get_default()
-        cursorlog("make_cursor: has-name=%s, has-cursor-types=%s, xscale=%s, yscale=%s, USE_LOCAL_CURSORS=%s",
+        cursorlog("make_cursor(%s) has-name=%s, has-cursor-types=%s, xscale=%s, yscale=%s, USE_LOCAL_CURSORS=%s",
+                  ellipsizer(cursor_data),
                   len(cursor_data)>=10, bool(cursor_types), self.xscale, self.yscale, USE_LOCAL_CURSORS)
         pixbuf = None
         if len(cursor_data)>=10 and cursor_types:
@@ -987,7 +983,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 cursorlog("scaling cursor from %ix%i to fixed OS size %ix%i", w, h, fw, fh)
                 cursor_pixbuf = pixbuf.scale_simple(fw, fh, GdkPixbuf.InterpType.BILINEAR)
                 xratio, yratio = w/fw, h/fh
-                x, y = iround(x/xratio), iround(y/yratio)
+                x, y = round(x/xratio), round(y/yratio)
         else:
             sx, sy, sw, sh = x, y, w, h
             #scale the cursors:
@@ -1003,8 +999,8 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 if cmaxh>0:
                     ratio = max(ratio, h/cmaxh)
                 cursorlog("clamping cursor size to %ix%i using ratio=%s", cmaxw, cmaxh, ratio)
-                sx, sy = iround(x/ratio), iround(y/ratio)
-                sw, sh = min(cmaxw, iround(w/ratio)), min(cmaxh, iround(h/ratio))
+                sx, sy = round(x/ratio), round(y/ratio)
+                sw, sh = min(cmaxw, round(w/ratio)), min(cmaxh, round(h/ratio))
             if sw!=w or sh!=h:
                 cursorlog("scaling cursor from %ix%i hotspot at %ix%i to %ix%i hotspot at %ix%i",
                           w, h, x, y, sw, sh, sx, sy)
@@ -1099,15 +1095,36 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             window.get_window().restack(other_window, above)  #Above=0
 
     def opengl_setup_failure(self, summary = "Xpra OpenGL Acceleration Failure", body=""):
+        OK = "0"
+        DISABLE = "1"
+        def notify_callback(event, nid, action_id, *args):
+            log("notify_callback(%s, %s, %s, %s)", event, nid, action_id, args)
+            if event=="notification-action" and nid==XPRA_OPENGL_NOTIFICATION_ID and action_id==DISABLE:
+                from xpra.platform.paths import get_user_conf_dirs
+                dirs = get_user_conf_dirs()
+                for d in dirs:
+                    conf_file = os.path.join(d, "xpra.conf")
+                    try:
+                        with open(conf_file, 'ab') as f:
+                            f.write(b"\n")
+                            f.write(b"# user chose to disable the opengl warning:\n")
+                            f.write(b"opengl=nowarn\n")
+                        log.info("OpenGL warning will be silenced from now on,")
+                        log.info(" '%s' has been updated", conf_file)
+                    except OSError:
+                        log("failed to create / append to config file '%s'", conf_file, exc_info=True)
         def delayed_notify():
-            if self.exit_code is None:
-                if OSX:
-                    #don't bother logging an error on MacOS,
-                    #OpenGL is being deprecated
-                    log.info(summary)
-                    log.info(body)
-                else:
-                    self.may_notify(XPRA_OPENGL_NOTIFICATION_ID, summary, body, icon_name="opengl")
+            if self.exit_code is not None:
+                return
+            if OSX:
+                #don't bother logging an error on MacOS,
+                #OpenGL is being deprecated
+                log.info(summary)
+                log.info(body)
+                return
+            actions = (OK, "OK", DISABLE, "Disable this warning")
+            self.may_notify(XPRA_OPENGL_NOTIFICATION_ID, summary, body, actions,
+                            icon_name="opengl", callback=notify_callback)
         #wait for the main loop to run:
         self.timeout_add(2*1000, delayed_notify)
 
@@ -1194,18 +1211,16 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             self.gl_max_viewport_dims = self.opengl_props.get("max-viewport-dims",
                                                               (self.gl_texture_size_limit, self.gl_texture_size_limit))
             driver_info = self.opengl_props.get("renderer") or self.opengl_props.get("vendor") or "unknown card"
+            def disablewarn(msg):
+                opengllog.warn("Warning: OpenGL is disabled:")
+                opengllog.warn(" %s", msg)
+                self.opengl_enabled = False
             if min(self.gl_max_viewport_dims)<4*1024:
-                opengllog.warn("Warning: OpenGL is disabled:")
-                opengllog.warn(" the maximum viewport size is too low: %s", self.gl_max_viewport_dims)
-                self.opengl_enabled = False
+                disablewarn("the maximum viewport size is too low: %s" % (self.gl_max_viewport_dims,))
             elif self.gl_texture_size_limit<4*1024:
-                opengllog.warn("Warning: OpenGL is disabled:")
-                opengllog.warn(" the texture size limit is too low: %s", self.gl_texture_size_limit)
-                self.opengl_enabled = False
+                disablewarn("the texture size limit is too low: %s" % (self.gl_texture_size_limit,))
             elif driver_info.startswith("SVGA3D") and os.environ.get("WAYLAND_DISPLAY"):
-                opengllog.warn("Warning: OpenGL is disabled:")
-                opengllog.warn(" SVGA3D driver is buggy under Wayland")
-                self.opengl_enabled = False
+                disablewarn("SVGA3D driver is buggy under Wayland")
             self.GLClientWindowClass.MAX_VIEWPORT_DIMS = self.gl_max_viewport_dims
             self.GLClientWindowClass.MAX_BACKING_DIMS = self.gl_texture_size_limit, self.gl_texture_size_limit
             mww, mwh = self.max_window_size
@@ -1277,7 +1292,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             if metadata.intget("transient-for", 0)>0:
                 log("not using opengl for transient-for window")
                 return False
-            if metadata.strget("content-type")=="text":
+            if metadata.strget("content-type", "").find("text")>=0:
                 return False
         if WIN32:
             #these checks can't be forced ('opengl_force')
@@ -1403,21 +1418,6 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
                 self.remote_clipboard_requests = 0
                 self.clipboard_notify(0)
             self.connect("clipboard-toggled", clipboard_toggled)
-            def loop_disabled_notify():
-                ch = self.clipboard_helper
-                if ch and ch.disabled_by_loop and self.notifier:
-                    icon = None
-                    try:
-                        from xpra.notifications.common import parse_image_path
-                        icon = parse_image_path(get_icon_filename("clipboard"))
-                    except ImportError:
-                        pass
-                    summary = "Clipboard Synchronization Error"
-                    body = "A synchronization loop has been detected,\n" + \
-                            "to prevent further issues clipboard synchronization has been disabled."
-                    self.notifier.show_notify("", self.tray, 0, "Xpra", 0, "", summary, body, [], {}, 10*10000, icon)
-                return False
-            self.timeout_add(5*1000, loop_disabled_notify)
         self.after_handshake(register_clipboard_toggled)
         if self.server_clipboard:
             #from now on, we will send a message to the server whenever the clipboard flag changes:
@@ -1437,7 +1437,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
         clipboardlog("clipboard_notify(%s) notification timer=%s", n, self.clipboard_notification_timer)
         self.cancel_clipboard_notification_timer()
         if n>0 and self.clipboard_enabled:
-            self.last_clipboard_notification = monotonic_time()
+            self.last_clipboard_notification = monotonic()
             tray.set_icon("clipboard")
             tray.set_tooltip("%s clipboard requests in progress" % n)
             tray.set_blinking(True)
@@ -1446,7 +1446,7 @@ class GTKXpraClient(GObjectXpraClient, UIXpraClient):
             #reset the tray icon,
             #but wait at least N seconds after the last clipboard transfer:
             N = 1
-            delay = int(max(0, 1000*(self.last_clipboard_notification+N-monotonic_time())))
+            delay = int(max(0, 1000*(self.last_clipboard_notification+N-monotonic())))
             def reset_tray_icon():
                 self.clipboard_notification_timer = None
                 tray = self.tray
