@@ -1,19 +1,23 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import math
 import os.path
+from time import monotonic
 from urllib.parse import unquote
-from cairo import OPERATOR_OVER, LINE_CAP_ROUND #pylint: disable=no-name-in-module
+from cairo import ( #pylint: disable=no-name-in-module
+    RectangleInt, Region,
+    OPERATOR_OVER, LINE_CAP_ROUND,
+    )
 from gi.repository import Gtk, Gdk, Gio
 
-from xpra.os_util import bytestostr, strtobytes, is_X11, monotonic_time, WIN32, OSX, POSIX
+from xpra.os_util import bytestostr, strtobytes, is_X11, WIN32, OSX, POSIX
 from xpra.util import (
-    AdHocStruct, typedict, envint, envbool, csv, first_time,
+    typedict, envint, envbool, csv, first_time, net_utf8,
     WORKSPACE_UNSET, WORKSPACE_ALL, WORKSPACE_NAMES, MOVERESIZE_DIRECTION_STRING, SOURCE_INDICATION_STRING,
     MOVERESIZE_CANCEL,
     MOVERESIZE_SIZE_TOPLEFT, MOVERESIZE_SIZE_TOP, MOVERESIZE_SIZE_TOPRIGHT,
@@ -30,6 +34,7 @@ from xpra.gtk_common.gtk_util import (
     WINDOW_EVENT_MASK,
     )
 from xpra.gtk_common.keymap import KEY_TRANSLATIONS
+from xpra.common import KeyEvent
 from xpra.client.client_window_base import ClientWindowBase
 from xpra.platform.gui import set_fullscreen_monitors, set_shaded
 from xpra.platform.gui import add_window_hooks, remove_window_hooks
@@ -109,7 +114,12 @@ WINDOW_OVERFLOW_TOP = envbool("XPRA_WINDOW_OVERFLOW_TOP", False)
 AWT_RECENTER = envbool("XPRA_AWT_RECENTER", True)
 UNDECORATED_TRANSIENT_IS_OR = envint("XPRA_UNDECORATED_TRANSIENT_IS_OR", 1)
 XSHAPE = envbool("XPRA_XSHAPE", True)
-LAZY_SHAPE = envbool("XPRA_LAZY_SHAPE", True)
+try:
+    from xpra.codecs.argb.argb import bit_to_rectangles
+except ImportError:
+    bit_to_rectangles = None
+LAZY_SHAPE = envbool("XPRA_LAZY_SHAPE", not callable(bit_to_rectangles))
+
 def parse_padding_colors(colors_str):
     padding_colors = 0, 0, 0
     if colors_str:
@@ -203,10 +213,6 @@ def wn(w):
     return WORKSPACE_NAMES.get(w, w)
 
 
-class GTKKeyEvent(AdHocStruct):
-    pass
-
-
 class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
 
     __common_gsignals__ = {
@@ -238,6 +244,7 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
         self._screen = -1
         self._frozen = False
         self._focus_latest = None
+        self._ondeiconify = []
         self.window_state_timer = None
         self.send_iconify_timer = None
         self.remove_pointer_overlay_timer = None
@@ -263,8 +270,8 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
         self.add(widget)
 
     def repaint(self, x, y, w, h):
-        #self.queue_draw_area(0, 0, *self._size)
         widget = self.drawing_area
+        #log("repaint%s widget=%s", (x, y, w, h), widget)
         if widget:
             widget.queue_draw_area(x, y, w, h)
 
@@ -286,6 +293,9 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
         def scroll(_w, event):
             self._do_scroll_event(event)
             return True
+        def configure_event(_w, event):
+            geomlog("widget configure_event: new size=%ix%i", event.width, event.height)
+        widget.connect("configure-event", configure_event)
         widget.connect("scroll-event", scroll)
         widget.connect("draw", self.draw_widget)
 
@@ -329,8 +339,6 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
 
     def drag_got_data_cb(self, wid, context, x, y, selection, info, time):
         draglog("drag_got_data_cb%s", (wid, context, x, y, selection, info, time))
-        #draglog("%s: %s", type(selection), dir(selection))
-        #draglog("%s: %s", type(context), dir(context))
         targets = list(x.name() for x in context.list_targets())
         actions = context.get_actions()
         def xid(w):
@@ -446,29 +454,37 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
         self.connect("focus-out-event", focus_out)
         if not self._override_redirect:
             self.connect("notify::has-toplevel-focus", self._focus_change)
+        def grab_broken(win, event):
+            grablog("grab_broken%s", (win, event))
+            self._client._window_with_grab = None
+        self.connect("grab-broken-event", grab_broken)
 
     def _focus_change(self, *args):
         assert not self._override_redirect
         htf = self.has_toplevel_focus()
         focuslog("%s focus_change%s has-toplevel-focus=%s, _been_mapped=%s", self, args, htf, self._been_mapped)
         if self._been_mapped:
-            self._client.update_focus(self._id, htf)
+            self._focus_latest = htf
+            self.send_latest_focus()
 
     def recheck_focus(self):
         self.recheck_focus_timer = 0
-        #we receive pairs of FocusOut + FocusIn following a keyboard grab,
-        #so we recheck the focus status via this timer to skip unnecessary churn
+        self.send_latest_focus()
+
+    def send_latest_focus(self):
         focused = self._client._focused
         focuslog("recheck_focus() wid=%i, focused=%s, latest=%s", self._id, focused, self._focus_latest)
-        hasfocus = focused==self._id
-        if hasfocus==self._focus_latest:
-            #we're already up to date
-            return
-        if not self._focus_latest:
-            self._client.window_ungrab()
-            self._client.update_focus(self._id, False)
+        if self._focus_latest:
+            self._focus()
         else:
-            self._client.update_focus(self._id, True)
+            self._unfocus()
+
+    def _focus(self):
+        super()._focus()
+
+    def _unfocus(self):
+        self._client.window_ungrab()
+        super()._unfocus()
 
     def cancel_focus_timer(self):
         rft = self.recheck_focus_timer
@@ -477,12 +493,12 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
             self.source_remove(rft)
 
     def schedule_recheck_focus(self):
+        #we receive pairs of FocusOut + FocusIn following a keyboard grab,
+        #so we recheck the focus status via this timer to skip unnecessary churn
         if FOCUS_RECHECK_DELAY<0:
             self.recheck_focus()
-            return
-        if self.recheck_focus_timer==0:
+        elif self.recheck_focus_timer==0:
             self.recheck_focus_timer = self.timeout_add(FOCUS_RECHECK_DELAY, self.recheck_focus)
-        return True
 
     def do_xpra_focus_out_event(self, event):
         focuslog("do_xpra_focus_out_event(%s)", event)
@@ -492,13 +508,15 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
                 focuslog("dropped NotifyInferior focus event")
                 return True
         self._focus_latest = False
-        return self.schedule_recheck_focus()
+        self.schedule_recheck_focus()
+        return True
 
     def do_xpra_focus_in_event(self, event):
         focuslog("do_xpra_focus_in_event(%s) been_mapped=%s", event, self._been_mapped)
         if self._been_mapped:
             self._focus_latest = True
-            return self.schedule_recheck_focus()
+            self.schedule_recheck_focus()
+        return True
 
 
     def init_max_window_size(self):
@@ -528,7 +546,7 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
 
 
     def is_awt(self, metadata) -> bool:
-        wm_class = metadata.get("class-instance")
+        wm_class = metadata.strtupleget("class-instance")
         return wm_class and len(wm_class)==2 and wm_class[0].startswith("sun-awt-X11")
 
     def _is_popup(self, metadata) -> bool:
@@ -536,8 +554,8 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
         if self._override_redirect:
             return True
         if UNDECORATED_TRANSIENT_IS_OR>0:
-            transient_for = metadata.get("transient-for", -1)
-            decorations = metadata.get("decorations", 0)
+            transient_for = metadata.intget("transient-for", -1)
+            decorations = metadata.intget("decorations", 0)
             if transient_for>0 and decorations<=0:
                 if UNDECORATED_TRANSIENT_IS_OR>1:
                     metalog("forcing POPUP type for window transient-for=%s", transient_for)
@@ -809,6 +827,17 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
         self.deiconify()
 
 
+    def deiconify(self):
+        for function in self._ondeiconify:
+            try:
+                function()
+            except Exception as e:
+                log.error("Error calling %s on %s during deiconification:", function, self)
+                log.error(" %s", e)
+        self._ondeiconify = []
+        Gtk.Window.deiconify(self)
+
+
     def window_state_updated(self, widget, event):
         statelog("%s.window_state_updated(%s, %s) changed_mask=%s, new_window_state=%s",
                  self, widget, repr(event), event.changed_mask, event.new_window_state)
@@ -921,12 +950,7 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
     def set_command(self, command):
         if not HAS_X11_BINDINGS:
             return
-        v = command
-        if not isinstance(command, str):
-            try:
-                v = v.decode("utf8")
-            except UnicodeDecodeError:
-                v = bytestostr(command)
+        v = net_utf8(command)
         def do_set_command():
             metalog("do_set_command() str(%s)='%r' (type=%s)", command, v, type(command))
             prop_set(self.get_window(), "WM_COMMAND", "latin1", v)
@@ -996,26 +1020,15 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
         d = ImageDraw.Draw(img)
         for x,y,w,h in rectangles:
             d.rectangle([x, y, x+w, y+h], fill=1)
-        img = img.resize((ww, wh))
+        shapelog("drawing complete")
+        img = img.resize((ww, wh), resample=Image.BICUBIC)
         shapelog("resized %s bitmap to window size %sx%s: %s", kind_name, ww, wh, img)
         #now convert back to rectangles...
-        rectangles = []
-        for y in range(wh):
-            #for debugging, this is very useful, but costly!
-            #shapelog("pixels[%3i]=%s", y, "".join([str(img.getpixel((x, y))) for x in range(ww)]))
-            x = 0
-            start = None
-            while x<ww:
-                #find first white pixel:
-                while x<ww and img.getpixel((x, y))==0:
-                    x += 1
-                start = x
-                #find next black pixel:
-                while x<ww and img.getpixel((x, y))!=0:
-                    x += 1
-                end = x
-                if start<end:
-                    rectangles.append((start, y, end-start, 1))
+        monodata = img.tobytes("raw", "1")
+        shapelog("got %i bytes", len(monodata))
+        #log.warn("monodata: %s (%i bytes) %ix%i", repr_ellipsized(monodata), len(monodata), ww, wh)
+        rectangles = bit_to_rectangles(monodata, ww, wh)
+        shapelog("back to rectangles")
         return rectangles
 
     def set_bypass_compositor(self, v):
@@ -1064,7 +1077,6 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
 
 
     def set_window_type(self, window_types):
-        pass
         hints = 0
         for window_type in window_types:
             #win32 workaround:
@@ -1130,6 +1142,18 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
                     self.set_size_constraints(self.size_constraints, self.max_window_size)
         self.when_realized("fullscreen", do_set_fullscreen)
 
+    def set_opaque_region(self, rectangles=()):
+        #gtk can only set a single region!
+        r = Region()
+        for rect in rectangles:
+            rect = RectangleInt(*self._client.srect(*rect))
+            r.union(Region(rect))
+        def do_set_region():
+            log("set_opaque_region(%s)", r)
+            self.get_window().set_opaque_region(r)
+        self.when_realized("set-opaque-region", do_set_region)
+
+
     def set_xid(self, xid):
         if not HAS_X11_BINDINGS:
             return
@@ -1289,7 +1313,6 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
             #which will look at the window metadata again
             workspacelog("workspace=%s will be set when the window is mapped", wn(workspace))
             return
-        workspace = workspace
         if workspace is not None:
             workspace = workspace & 0xffffffff
         desktop = self.get_desktop_workspace()
@@ -1361,12 +1384,12 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
         if gdkwin:
             d = gdkwin.get_display()
             if d:
-                d.keyboard_ungrab(0)
+                d.keyboard_ungrab(Gdk.CURRENT_TIME)
         return True
 
     def keyboard_grab(self, *args):
         grablog("keyboard_grab%s", args)
-        r = Gdk.keyboard_grab(self.get_window(), True, 0)
+        r = Gdk.keyboard_grab(self.get_window(), True, Gdk.CURRENT_TIME)
         self._client.keyboard_grabbed = r==Gdk.GrabStatus.SUCCESS
         grablog("keyboard_grab%s Gdk.keyboard_grab(%s, True)=%s, keyboard_grabbed=%s",
                 args, self.get_window(), GRAB_STATUS_STRING.get(r), self._client.keyboard_grabbed)
@@ -1388,7 +1411,7 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
                       em.POINTER_MOTION_HINT_MASK |
                       em.ENTER_NOTIFY_MASK |
                       em.LEAVE_NOTIFY_MASK)
-        r = Gdk.pointer_grab(gdkwin, True, event_mask, gdkwin, None, 0)
+        r = Gdk.pointer_grab(gdkwin, True, event_mask, gdkwin, None, Gdk.CURRENT_TIME)
         self._client.pointer_grabbed = r==Gdk.GrabStatus.SUCCESS
         grablog("pointer_grab%s Gdk.pointer_grab(%s, True)=%s, pointer_grabbed=%s",
                 args, self.get_window(), GRAB_STATUS_STRING.get(r), self._client.pointer_grabbed)
@@ -1401,7 +1424,7 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
         if gdkwin:
             d = gdkwin.get_display()
             if d:
-                d.pointer_ungrab(0)
+                d.pointer_ungrab(Gdk.CURRENT_TIME)
         return True
 
     def toggle_pointer_grab(self):
@@ -1648,7 +1671,8 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
                     }.get(direction)
                 geomlog("edge(%s)=%s", MOVERESIZE_DIRECTION_STRING.get(direction), edge)
                 if direction is not None:
-                    self.begin_resize_drag(edge, button, x, y, 0)
+                    etime = Gtk.get_current_event_time()
+                    self.begin_resize_drag(edge, button, x, y, etime)
         else:
             #handle it ourselves:
             #use window coordinates (which include decorations)
@@ -1745,7 +1769,7 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
         context.set_line_cap(LINE_CAP_ROUND)
         context.translate(w/2, h/2)
         from xpra.client.spinner import cv
-        count = int(monotonic_time()*4.0)
+        count = int(monotonic()*4.0)
         for i in range(8):      #8 lines
             context.set_source_rgba(0, 0, 0, cv.trs[count%8][i])
             context.move_to(0.0, -dim/4.0)
@@ -1775,7 +1799,9 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
         #(as this also honoured correctly with CSD,
         # whereas set_geometry_hints is not..)
         minw, minh = self.size_constraints.intpair("minimum-size", (0, 0))
-        self.drawing_area.set_size_request(minw, minh)
+        w, h = self._client.sp(minw, minh)
+        geomlog("do_map_event %s.set_size_request%s", self.drawing_area, (minw, minh))
+        self.drawing_area.set_size_request(w, h)
 
     def process_map_event(self):
         x, y, w, h = self.get_drawing_area_geometry()
@@ -1810,13 +1836,13 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
         cx = self._client.cx
         cy = self._client.cy
         sx, sy, sw, sh = cx(x), cy(y), cx(w), cy(h)
+        if self._backing is None:
+            #we may have cleared the backing, so we must re-create one:
+            self._set_backing_size(w, h)
         packet = ["map-window", self._id, sx, sy, sw, sh, props, state]
         self.send(*packet)
         self._pos = (x, y)
         self._size = (w, h)
-        if self._backing is None:
-            #we may have cleared the backing, so we must re-create one:
-            self._set_backing_size(w, h)
         if not self._override_redirect:
             htf = self.has_toplevel_focus()
             focuslog("mapped: has-toplevel-focus=%s", htf)
@@ -1833,6 +1859,12 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
         return frame
 
 
+    def may_send_client_properties(self):
+        #if there are client properties the server should know about,
+        #we currently have no other way to send them to the server:
+        if self._client_properties:
+            self.send_configure_event(True)
+
     def send_configure(self):
         self.send_configure_event()
 
@@ -1851,7 +1883,6 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
         ox, oy = self._pos
         dx, dy = x-ox, y-oy
         self._pos = (x, y)
-        self.send_configure_event(skip_geometry)
         if dx!=0 or dy!=0:
             #window has moved, also move any child OR window:
             for window in self._override_redirect_windows:
@@ -1861,8 +1892,9 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
                 self._size, (w, h), self._backing, self._iconified)
         self._size = (w, h)
         self._set_backing_size(w, h)
+        self.send_configure_event(skip_geometry)
         if self._backing and not self._iconified:
-            geomlog("configure event: size unchanged, queueing redraw")
+            geomlog("configure event: queueing redraw")
             self.repaint(0, 0, w, h)
 
     def send_configure_event(self, skip_geometry=False):
@@ -1891,7 +1923,7 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
         if self.is_OR():
             pwid = -1
         packet.append(pwid)
-        packet.append(self._client.get_mouse_position())
+        packet.append(self.get_mouse_position())
         packet.append(self._client.get_current_modifiers())
         geomlog("%s", packet)
         self.send(*packet)
@@ -1905,10 +1937,12 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
         if max(bw, bh)>=32000:
             raise Exception("invalid window backing size %ix%i" % (bw, bh))
         if b:
+            prev_render_size = b.render_size
             b.init(ww, wh, bw, bh)
+            if prev_render_size!=b.render_size:
+                self._client_properties["encoding.render-size"] = b.render_size
         else:
             self.new_backing(bw, bh)
-        self._client_properties["encoding.render-size"] = b.render_size
 
     def resize(self, w, h, resize_counter=0):
         ww, wh = self.get_size()
@@ -1928,6 +1962,7 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
         geomlog("backing offsets=%s, window offset=%s", self._backing.offsets, self.window_offset)
         self._set_backing_size(w, h)
         self.repaint(0, 0, ww, wh)
+        self.may_send_client_properties()
 
     def center_backing(self, w, h):
         ww, wh = self.get_size()
@@ -2027,6 +2062,8 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
         window.move_resize(ax, ay, w, h)
         #re-init the backing with the new size
         self._set_backing_size(w, h)
+        self.repaint(0, 0, w, h)
+        self.may_send_client_properties()
 
 
     def noop_destroy(self):
@@ -2059,6 +2096,14 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
         return True
 
 
+    def get_mouse_position(self):
+        #this method is used on some platforms
+        #to get the pointer position for events that don't include it
+        #(ie: wheel events)
+        x, y = self._client.get_raw_mouse_position()
+        return self._offset_pointer(x, y)
+
+
     def _offset_pointer(self, x, y):
         if self.window_offset:
             x -= self.window_offset[0]
@@ -2071,16 +2116,25 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
     def _get_relative_pointer(self, event):
         return event.x, event.y
 
-    def _pointer_modifiers(self, event):
+    def get_pointer_data(self, event):
         x, y = self._get_pointer(event)
-        rx, ry = self._get_relative_pointer(event)
-        #adjust for window offset:
-        pointer = self._offset_pointer(x, y)
-        relative_pointer = self._client.cp(rx, ry)
+        rx, ry = self._get_relative_pointer(event)#
+        return self.adjusted_pointer_data(x, y, rx, ry)
+
+    def adjusted_pointer_data(self, x, y, rx=0, ry=0):
+        #regular pointer coordinates are translated and scaled,
+        #relative coordinates are scaled only:
+        data = self._offset_pointer(x, y)
+        if self._client.server_pointer_relative:
+            data = list(data) + list(self._client.cp(rx, ry))
+        return data
+
+    def _pointer_modifiers(self, event):
+        pointer_data = self.get_pointer_data(event)
         #FIXME: state is used for both mods and buttons??
         modifiers = self._client.mask_to_names(event.state)
         buttons = self._event_buttons(event)
-        v = pointer, relative_pointer, modifiers, buttons
+        v = pointer_data, modifiers, buttons
         mouselog("pointer_modifiers(%s)=%s (x_root=%s, y_root=%s, window_offset=%s)",
                  event, v, event.x_root, event.y_root, self.window_offset)
         return v
@@ -2093,7 +2147,7 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
         keycode = event.hardware_keycode
         keyname = Gdk.keyval_name(keyval)
         keyname = KEY_TRANSLATIONS.get((keyname, keyval, keycode), keyname)
-        key_event = GTKKeyEvent()
+        key_event = KeyEvent()
         key_event.modifiers = self._client.mask_to_names(event.state)
         key_event.keyname = keyname or ""
         key_event.keyval = keyval or 0
@@ -2128,6 +2182,11 @@ class GTKClientWindowBase(ClientWindowBase, Gtk.Window):
     def _do_scroll_event(self, event):
         if self._client.readonly:
             return
+        if event.direction==Gdk.ScrollDirection.SMOOTH:
+            mouselog("smooth scroll event: %s", event)
+            pointer = self.get_pointer_data(event)
+            self._client.wheel_event(self._id, event.delta_x, -event.delta_y, pointer)
+            return
         button_mapping = GDK_SCROLL_MAP.get(event.direction, -1)
         mouselog("do_scroll_event device=%s, direction=%s, button_mapping=%s",
                  self._device_info(event), event.direction, button_mapping)
