@@ -1,22 +1,23 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
+from time import monotonic
 
 from xpra.exit_codes import EXIT_INTERNAL_ERROR
 from xpra.platform.features import REINIT_WINDOWS
 from xpra.platform.gui import (
+    get_vrefresh,
     get_antialias_info, get_icc_info, get_display_icc_info, show_desktop, get_cursor_size,
     get_xdpi, get_ydpi, get_number_of_desktops, get_desktop_names, get_wm_name,
     )
 from xpra.scripts.main import check_display
 from xpra.scripts.config import FALSE_OPTIONS
 from xpra.net.common import MAX_PACKET_SIZE
-from xpra.os_util import monotonic_time
 from xpra.util import (
-    iround, envint, envfloat, envbool, log_screen_sizes, engs, flatten_dict, typedict,
+    envint, envfloat, envbool, log_screen_sizes, flatten_dict, typedict,
     XPRA_SCALING_NOTIFICATION_ID,
     )
 from xpra.client.mixins.stub_client_mixin import StubClientMixin
@@ -37,21 +38,21 @@ SYNC_ICC = envbool("XPRA_SYNC_ICC", True)
 
 
 def r4cmp(v, rounding=1000.0):    #ignore small differences in floats for scale values
-    return iround(v*rounding)
+    return round(v*rounding)
 def fequ(v1, v2):
     return r4cmp(v1)==r4cmp(v2)
 
 
-"""
-Utility superclass for clients that handle a desktop / display
-Adds client-side scaling handling
-"""
 class DisplayClient(StubClientMixin):
+    """
+    Utility mixin for clients that handle a desktop / display
+    Adds client-side scaling handling
+    """
     __signals__ = ["scaling-changed"]
 
     def __init__(self):
         check_display()
-        StubClientMixin.__init__(self)
+        super().__init__()
         self.dpi = 0
         self.can_scale = False
         self.initial_scaling = 1, 1
@@ -74,6 +75,7 @@ class DisplayClient(StubClientMixin):
         self.desktop_scaling = opts.desktop_scaling
         self.dpi = int(opts.dpi)
         self.can_scale = opts.desktop_scaling not in FALSE_OPTIONS
+        scalinglog("can_scale(%s)=%s", opts.desktop_scaling, self.can_scale)
         if self.can_scale:
             self.parse_scaling(opts.desktop_scaling)
 
@@ -82,6 +84,7 @@ class DisplayClient(StubClientMixin):
         from xpra.client.scaling_parser import parse_scaling
         self.initial_scaling = parse_scaling(desktop_scaling, root_w, root_h, MIN_SCALING, MAX_SCALING)
         self.xscale, self.yscale = self.initial_scaling
+        scalinglog("scaling(%s)=%s", self.initial_scaling, (self.xscale, self.yscale))
 
     def cleanup(self):
         ssct = self.screen_size_change_timer
@@ -112,32 +115,33 @@ class DisplayClient(StubClientMixin):
         caps = {
             "randr_notify"  : True,
             "show-desktop"  : True,
+            "vrefresh"      : self.get_vrefresh(),
             }
         wm_name = get_wm_name()
         if wm_name:
-            caps["wm_name"] = wm_name.encode("utf8")
+            caps["wm_name"] = wm_name
 
         self._last_screen_settings = self.get_screen_settings()
-        root_w, root_h, sss, ndesktops, desktop_names, u_root_w, u_root_h, _, _ = self._last_screen_settings
+        root_w, root_h, sss, ndesktops, desktop_names, u_root_w, u_root_h = self._last_screen_settings[:7]
         if u_root_w and u_root_h:
             caps["desktop_size"] = self.cp(u_root_w, u_root_h)
         if ndesktops:
             caps["desktops"] = ndesktops
-            caps["desktop.names"] = tuple(x.encode("utf8") for x in desktop_names)
+            caps["desktop.names"] = tuple(desktop_names)
 
         ss = self.get_screen_sizes()
         self._current_screen_sizes = ss
 
-        log.info(" desktop size is %sx%s with %s screen%s:", u_root_w, u_root_h, len(ss), engs(ss))
+        log.info(" desktop size is %sx%s:", u_root_w, u_root_h)
         log_screen_sizes(u_root_w, u_root_h, ss)
         if self.xscale!=1 or self.yscale!=1:
             caps["screen_sizes.unscaled"] = ss
             caps["desktop_size.unscaled"] = u_root_w, u_root_h
             root_w, root_h = self.cp(u_root_w, u_root_h)
             if fequ(self.xscale, self.yscale):
-                sinfo = "%i%%" % iround(self.xscale*100)
+                sinfo = "%i%%" % round(self.xscale*100)
             else:
-                sinfo = "%i%% x %i%%" % (iround(self.xscale*100), iround(self.yscale*100))
+                sinfo = "%i%% x %i%%" % (round(self.xscale*100), round(self.yscale*100))
             scaled_up = u_root_w>root_w or u_root_h>root_h
             log.info(" %sscaled to %s, virtual screen size: %ix%i",
                      "up" if scaled_up else "down", sinfo, root_w, root_h)
@@ -160,7 +164,7 @@ class DisplayClient(StubClientMixin):
         dpi = 0
         if self.dpi>0:
             #scale it:
-            dpi = iround((self.cx(self.dpi) + self.cy(self.dpi))/2.0)
+            dpi = round((self.cx(self.dpi) + self.cy(self.dpi))/2.0)
         else:
             #not supplied, use platform detection code:
             #platforms may also provide per-axis dpi (later win32 versions do)
@@ -170,7 +174,7 @@ class DisplayClient(StubClientMixin):
             if xdpi>0 and ydpi>0:
                 xdpi = self.cx(xdpi)
                 ydpi = self.cy(ydpi)
-                dpi = iround((xdpi+ydpi)/2.0)
+                dpi = round((xdpi+ydpi)/2.0)
                 caps = {
                     "x"    : xdpi,
                     "y"    : ydpi,
@@ -187,8 +191,12 @@ class DisplayClient(StubClientMixin):
             "values"    : (int(1000*self.xscale), int(1000*self.yscale)),
             }
 
+    def get_vrefresh(self):
+        return get_vrefresh()
+
     def get_screen_caps(self) -> dict:
         caps = {
+            "vrefresh"      : self.get_vrefresh(),
             "antialias"    : get_antialias_info(),
             "cursor" : {
                 "size"  : int(2*get_cursor_size()/(self.xscale+self.yscale)),
@@ -232,7 +240,7 @@ class DisplayClient(StubClientMixin):
         self.server_display = c.strget("display")
         self.server_desktop_size = c.intpair("desktop_size")
         log("server desktop size=%s", self.server_desktop_size)
-        self.server_max_desktop_size = c.intpair("max_desktop_size")
+        self.server_max_desktop_size = c.intpair("max_desktop_size", (2**15, 2**15))
         self.server_actual_desktop_size = c.intpair("actual_desktop_size")
         log("server actual desktop size=%s", self.server_actual_desktop_size)
         self.server_randr = c.boolget("resize_screen")
@@ -266,12 +274,15 @@ class DisplayClient(StubClientMixin):
                 log.warn("Server's virtual screen is too small")
                 log.warn(" server: %sx%s vs client: %sx%s", avail_w, avail_h, self.cx(root_w), self.cy(root_h))
                 log.warn(" you may see strange behavior,")
-                log.warn(" please see https://xpra.org/trac/wiki/Xdummy#Configuration")
+                log.warn(" please see https://github.com/Xpra-org/xpra/blob/master/docs/Usage/Xdummy.md")
         #now that we have the server's screen info, allow scale changes:
         self.scale_change_embargo = 0
         self.set_max_packet_size()
 
     def set_max_packet_size(self):
+        p = self._protocol
+        if not p or p.TYPE!="xpra":
+            return
         root_w, root_h = self.cp(*self.get_root_size())
         maxw, maxh = root_w, root_h
         try:
@@ -290,12 +301,10 @@ class DisplayClient(StubClientMixin):
         #max packet size to accomodate
         # a full screen RGBX (32 bits) uncompressed image
         # also with enough headroom for some metadata (4k)
-        p = self._protocol
-        if p:
-            #we can't assume to have a real ClientConnection object:
-            p.max_packet_size = max(MAX_PACKET_SIZE, maxw*maxh*4 + 4*1024)
-            p.abs_max_packet_size = maxw*maxh*4*4 + 4*1024
-            log("maximum packet size set to %i", p.max_packet_size)
+        #we can't assume to have a real ClientConnection object:
+        p.max_packet_size = max(MAX_PACKET_SIZE, maxw*maxh*4 + 4*1024)
+        p.abs_max_packet_size = maxw*maxh*4*4 + 4*1024
+        log("maximum packet size set to %i", p.max_packet_size)
 
 
     def has_transparency(self) -> bool:
@@ -355,7 +364,7 @@ class DisplayClient(StubClientMixin):
         def mint(v):
             #prefer int over float,
             #and even tolerate a 0.1% difference to get it:
-            if iround(v)*1000==iround(v*1000):
+            if round(v)*1000==round(v*1000):
                 return int(v)
             return v
         self.xscale = mint(x)
@@ -388,10 +397,10 @@ class DisplayClient(StubClientMixin):
         return v*self.yscale
     def sx(self, v) -> int:
         """ convert X coordinate from server to client """
-        return iround(self.fsx(v))
+        return round(self.fsx(v))
     def sy(self, v) -> int:
         """ convert Y coordinate from server to client """
-        return iround(self.fsy(v))
+        return round(self.fsy(v))
     def srect(self, x, y, w, h):
         """ convert rectangle coordinates from server to client """
         return self.sx(x), self.sy(y), self.sx(w), self.sy(h)
@@ -401,10 +410,10 @@ class DisplayClient(StubClientMixin):
 
     def cx(self, v) -> int:
         """ convert X coordinate from client to server """
-        return iround(v/self.xscale)
+        return round(v/self.xscale)
     def cy(self, v) -> int:
         """ convert Y coordinate from client to server """
-        return iround(v/self.yscale)
+        return round(v/self.yscale)
     def crect(self, x, y, w, h):
         """ convert rectangle coordinates from client to server """
         return self.cx(x), self.cy(y), self.cx(w), self.cy(h)
@@ -435,7 +444,7 @@ class DisplayClient(StubClientMixin):
         delay = 1000
         #if we are suspending, wait longer:
         #(better chance that the suspend-resume cycle will have completed)
-        if self._suspended_at>0 and self._suspended_at-monotonic_time()<5*1000:
+        if self._suspended_at>0 and self._suspended_at-monotonic()<5*1000:
             delay = 5*1000
         self.screen_size_change_timer = self.timeout_add(delay, self.do_process_screen_size_change)
 
@@ -472,7 +481,9 @@ class DisplayClient(StubClientMixin):
         xdpi = self.cx(xdpi)
         ydpi = self.cy(ydpi)
         log("get_screen_settings() scaled: xdpi=%s, ydpi=%s", xdpi, ydpi)
-        return (root_w, root_h, sss, ndesktops, desktop_names, u_root_w, u_root_h, xdpi, ydpi)
+        vrefresh = self.get_vrefresh()
+        log("get_screen_settings() vrefresh=%s", vrefresh)
+        return (root_w, root_h, sss, ndesktops, desktop_names, u_root_w, u_root_h, xdpi, ydpi, vrefresh)
 
     def update_screen_size(self):
         self.screen_size_change_timer = None
@@ -483,7 +494,7 @@ class DisplayClient(StubClientMixin):
             log("screen size unchanged")
             return
         root_w, root_h, sss = screen_settings[:3]
-        log.info("sending updated screen size to server: %sx%s with %s screens", root_w, root_h, len(sss))
+        log.info("sending updated screen size to server: %sx%s", root_w, root_h)
         log_screen_sizes(root_w, root_h, sss)
         if self.server_desktop_size:
             self.send("desktop_size", *screen_settings)
@@ -539,7 +550,7 @@ class DisplayClient(StubClientMixin):
         if self.screen_size_change_timer:
             scalinglog("scale_change(%s, %s) screen size change is already pending", xchange, ychange)
             return
-        if monotonic_time()<self.scale_change_embargo:
+        if monotonic()<self.scale_change_embargo:
             scalinglog("scale_change(%s, %s) screen size change not permitted during embargo time - try again",
                        xchange, ychange)
             return
@@ -582,18 +593,18 @@ class DisplayClient(StubClientMixin):
 
     def scale_reinit(self, xchange=1.0, ychange=1.0):
         #wait at least one second before changing again:
-        self.scale_change_embargo = monotonic_time()+SCALING_EMBARGO_TIME
+        self.scale_change_embargo = monotonic()+SCALING_EMBARGO_TIME
         if fequ(self.xscale, self.yscale):
-            scalinglog.info("setting scaling to %i%%:", iround(100*self.xscale))
+            scalinglog.info("setting scaling to %i%%:", round(100*self.xscale))
         else:
-            scalinglog.info("setting scaling to %i%% x %i%%:", iround(100*self.xscale), iround(100*self.yscale))
+            scalinglog.info("setting scaling to %i%% x %i%%:", round(100*self.xscale), round(100*self.yscale))
         self.update_screen_size()
         #re-initialize all the windows with their new size
         def new_size_fn(w, h):
             minx, miny = 16384, 16384
             if self.max_window_size!=(0, 0):
                 minx, miny = self.max_window_size
-            return max(1, min(minx, iround(w*xchange))), max(1, min(miny, iround(h*ychange)))
+            return max(1, min(minx, round(w*xchange))), max(1, min(miny, round(h*ychange)))
         self.resize_windows(new_size_fn)
         self.reinit_window_icons()
         self.emit("scaling-changed")
