@@ -1,12 +1,12 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2019 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2010-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
 
-from xpra.codecs.codec_constants import PREFERRED_ENCODING_ORDER, PROBLEMATIC_ENCODINGS
+from xpra.codecs.codec_constants import PREFERRED_ENCODING_ORDER
 from xpra.codecs.loader import load_codec, codec_versions, has_codec, get_codec
 from xpra.codecs.video_helper import getVideoHelper, NO_GFX_CSC_OPTIONS
 from xpra.scripts.config import parse_bool_or_int
@@ -28,13 +28,44 @@ SCROLL_ENCODING = envbool("XPRA_SCROLL_ENCODING", True)
 DEFAULT_ENCODINGS = os.environ.get("XPRA_DEFAULT_ENCODINGS", "rgb32,rgb24,jpeg,png").split(",")
 
 
-"""
-Mixin for adding encodings to a client
-"""
+def get_core_encodings():
+    """
+        This method returns the actual encodings supported.
+        ie: ["rgb24", "vp8", "webp", "png", "png/L", "png/P", "jpeg", "h264", "vpx"]
+        It is often overriden in the actual client class implementations,
+        where extra encodings can be added (generally just 'rgb32' for transparency),
+        or removed if the toolkit implementation class is more limited.
+    """
+    #we always support rgb:
+    core_encodings = ["rgb24", "rgb32"]
+    for codec in ("dec_pillow", "dec_webp", "dec_jpeg"):
+        if has_codec(codec):
+            c = get_codec(codec)
+            encs = c.get_encodings()
+            log("%s.get_encodings()=%s", codec, encs)
+            for e in encs:
+                if e not in core_encodings:
+                    core_encodings.append(e)
+    if SCROLL_ENCODING:
+        core_encodings.append("scroll")
+    #we enable all the video decoders we know about,
+    #what will actually get used by the server will still depend on the csc modes supported
+    video_decodings = getVideoHelper().get_decodings()
+    log("video_decodings=%s", video_decodings)
+    for encoding in video_decodings:
+        if encoding not in core_encodings:
+            core_encodings.append(encoding)
+    #remove duplicates and use prefered encoding order:
+    return [x for x in PREFERRED_ENCODING_ORDER if x in set(core_encodings)]
+
+
 class Encodings(StubClientMixin):
+    """
+    Mixin for adding encodings to a client
+    """
 
     def __init__(self):
-        StubClientMixin.__init__(self)
+        super().__init__()
         self.allowed_encodings = []
         self.encoding = None
         self.quality = -1
@@ -46,7 +77,6 @@ class Encodings(StubClientMixin):
 
         self.server_encodings = []
         self.server_core_encodings = []
-        self.server_encodings_problematic = PROBLEMATIC_ENCODINGS
         self.server_encodings_with_speed = ()
         self.server_encodings_with_quality = ()
         self.server_encodings_with_lossless_mode = ()
@@ -68,11 +98,14 @@ class Encodings(StubClientMixin):
         self.min_speed = opts.min_speed
         load_codec("dec_pillow")
         ae = self.allowed_encodings
+        if "png" in ae:
+            #try to load the fast png decoder:
+            load_codec("dec_spng")
         if "jpeg" in ae:
-            #try to load the fast jpeg encoder:
+            #try to load the fast jpeg decoder:
             load_codec("dec_jpeg")
         if "webp" in ae:
-            #try to load the fast webp encoder:
+            #try to load the fast webp decoder:
             load_codec("dec_webp")
         vh = getVideoHelper()
         vh.set_modules(video_decoders=opts.video_decoders, csc_modules=opts.csc_modules or NO_GFX_CSC_OPTIONS)
@@ -124,15 +157,13 @@ class Encodings(StubClientMixin):
         updict(caps, "encoding",        self.get_encodings_caps())
         return caps
 
-    def parse_server_capabilities(self, caps : typedict) -> bool:
-        self._parse_server_capabilities(caps)
+    def parse_server_capabilities(self, c : typedict) -> bool:
+        self._parse_server_capabilities(c)
         return True
 
     def _parse_server_capabilities(self, c):
         self.server_encodings = c.strtupleget("encodings", DEFAULT_ENCODINGS)
         self.server_core_encodings = c.strtupleget("encodings.core", self.server_encodings)
-        #server is telling us to try to avoid those:
-        self.server_encodings_problematic = c.strtupleget("encodings.problematic", PROBLEMATIC_ENCODINGS)
         #old servers only supported x264:
         self.server_encodings_with_speed = c.strtupleget("encodings.with_speed", ("h264",))
         self.server_encodings_with_quality = c.strtupleget("encodings.with_quality", ("jpeg", "webp", "h264"))
@@ -203,6 +234,10 @@ class Encodings(StubClientMixin):
                 full_csc_modes["webp"] = ("BGRX", "BGRA", "RGBX", "RGBA")
             else:
                 full_csc_modes["webp"] = ("BGRX", "BGRA", )
+        if has_codec("dec_jpeg") or has_codec("dec_pillow"):
+            full_csc_modes["jpeg"] = ("BGRX", "BGRA", "YUV420P")
+        if has_codec("dec_jpeg"):
+            full_csc_modes["jpega"] = ("BGRA", "RGBA", )
         log("supported full csc_modes=%s", full_csc_modes)
         caps["full_csc_modes"] = full_csc_modes
 
@@ -255,36 +290,7 @@ class Encodings(StubClientMixin):
         return e
 
     def get_core_encodings(self):
-        """
-            This method returns the actual encodings supported.
-            ie: ["rgb24", "vp8", "webp", "png", "png/L", "png/P", "jpeg", "h264", "vpx"]
-            It is often overriden in the actual client class implementations,
-            where extra encodings can be added (generally just 'rgb32' for transparency),
-            or removed if the toolkit implementation class is more limited.
-        """
-        #we always support rgb:
-        core_encodings = ["rgb24", "rgb32"]
-        for codec in ("dec_pillow", "dec_webp", "dec_jpeg"):
-            if has_codec(codec):
-                c = get_codec(codec)
-                encs = c.get_encodings()
-                log("%s.get_encodings()=%s", codec, encs)
-                for e in encs:
-                    if e not in core_encodings:
-                        core_encodings.append(e)
-        if SCROLL_ENCODING:
-            core_encodings.append("scroll")
-        #we enable all the video decoders we know about,
-        #what will actually get used by the server will still depend on the csc modes supported
-        video_decodings = getVideoHelper().get_decodings()
-        log("video_decodings=%s", video_decodings)
-        for encoding in video_decodings:
-            if encoding not in core_encodings:
-                core_encodings.append(encoding)
-        #remove duplicates and use prefered encoding order:
-        core_encodings = [x for x in PREFERRED_ENCODING_ORDER
-                          if x in set(core_encodings) and x in self.allowed_encodings]
-        return core_encodings
+        return [x for x in get_core_encodings() if x in self.allowed_encodings]
 
     def set_encoding(self, encoding):
         log("set_encoding(%s)", encoding)
