@@ -8,12 +8,13 @@
 #pylint: disable=no-member
 
 import os
+from time import monotonic
 from threading import RLock
 import pycuda               #@UnresolvedImport
 from pycuda import driver   #@UnresolvedImport
 
 from xpra.util import engs, print_nested_dict, envint, csv, first_time
-from xpra.os_util import monotonic_time, load_binary_file
+from xpra.os_util import load_binary_file
 from xpra.log import Logger
 
 log = Logger("cuda")
@@ -191,7 +192,7 @@ def init_all_devices():
     if disabled_gpus is True or enabled_gpus==[]:
         log("all devices are disabled!")
         return DEVICES
-    log("init_all_devices() enabled: %s, disabled: %s", csv(enabled_gpus), csv(disabled_gpus))
+    log("init_all_devices() enabled: %s, disabled: %s", csv(enabled_gpus), csv(disabled_gpus) or "none")
     if not driver_init():
         return DEVICES
     ngpus = driver.Device.count()
@@ -301,7 +302,7 @@ def select_device(preferred_device_id=-1, min_compute=0):
     log("select_device(%s, %s)", preferred_device_id, min_compute)
     for device_id in (preferred_device_id, get_pref("device-id")):
         if device_id is not None and device_id>=0:
-            dct = load_device(device_id)
+            dct = make_device_context(device_id)
             if dct:
                 device, context, tpct = dct
                 context.pop()
@@ -363,7 +364,7 @@ def select_best_free_memory(min_compute=0):
         log("will test %s device%s from %s list: %s", len(device_list), engs(device_list), list_name, device_list)
         for device_id in device_list:
             context = None
-            dct = load_device(device_id)
+            dct = make_device_context(device_id)
             if not dct:
                 continue
             try:
@@ -396,19 +397,25 @@ def select_best_free_memory(min_compute=0):
 def load_device(device_id):
     log("load_device(%i)", device_id)
     try:
-        device = driver.Device(device_id)
+        return driver.Device(device_id)
     except Exception as e:
         log("load_device(%s)", device_id, exc_info=True)
         log.error("Error: allocating CUDA device %s", device_id)
         log.error(" %s", e)
+    return None
+
+def make_device_context(device_id):
+    log("make_device_context(%i)", device_id)
+    device = load_device(device_id)
+    if not device:
         return None
-    log("select_device: testing device %s: %s", device_id, device_info(device))
+    log("make_device_context(%i) device_info=%s", device_id, device_info(device))
     cf = driver.ctx_flags
     flags = cf.SCHED_YIELD | cf.MAP_HOST
     try:
         context = device.make_context(flags=flags)
     except Exception as e:
-        log("%s.make_context(%s)", flags, exc_info=True)
+        log("%s.make_context(%s)", device, flags, exc_info=True)
         log.error("Error: cannot create CUDA context for device %s", device_id)
         log.error(" %s", e)
         return None
@@ -428,7 +435,9 @@ def get_device_context(options):
 
 
 class cuda_device_context:
+    __slots__ = ("device_id", "device", "context", "lock")
     def __init__(self, device_id, device):
+        assert device, "no cuda device"
         self.device_id = device_id
         self.device = device
         self.context = None
@@ -441,27 +450,42 @@ class cuda_device_context:
     def __enter__(self):
         assert self.lock.acquire(False), "failed to acquire cuda device lock"
         if not self.context:
-            start = monotonic_time()
+            start = monotonic()
             cf = driver.ctx_flags
             self.context = self.device.make_context(flags=cf.SCHED_YIELD | cf.MAP_HOST)
-            end = monotonic_time()
+            end = monotonic()
             self.context.pop()
             log("cuda context allocation took %ims", 1000*(end-start))
         self.context.push()
+        return self.context
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         c = self.context
         if c:
             c.pop()
         self.lock.release()
+        #except driver.LogicError as e:
+        #log.warn("Warning: PyCUDA %s", e)
+        #self.clean()
+        #self.init_cuda()
+
 
     def __repr__(self):
         return "cuda_device_context(%i - %s)" % (self.device_id, self.lock._is_owned())
 
     def get_info(self):
-        return {
+        info = {
             "id" : self.device_id,
+            "device" : {
+                "name"         : self.device.name(),
+                "pci_bus_id"   : self.device.pci_bus_id(),
+                "memory"       : int(self.device.total_memory()//1024//1024),
+                },
             }
+        if self.context:
+            info["api_version"] = self.context.get_api_version()
+        return info
+
 
     def __del__(self):
         self.free()
@@ -544,7 +568,7 @@ CUDA_ERRORS_INFO = {
 
 #cache kernel fatbin files:
 KERNELS = {}
-def get_CUDA_function(device_id, function_name):
+def get_CUDA_function(function_name):
     """
         Returns the compiled kernel for the given device
         and kernel key.
@@ -554,7 +578,7 @@ def get_CUDA_function(device_id, function_name):
     if data is None:
         from xpra.platform.paths import get_resources_dir
         cubin_file = os.path.join(get_resources_dir(), "cuda", "%s.fatbin" % function_name)
-        log("get_CUDA_function(%s, %s) cubin file=%s", device_id, function_name, cubin_file)
+        log("get_CUDA_function(%s) cubin file=%s", function_name, cubin_file)
         data = load_binary_file(cubin_file)
         if not data:
             log.error("Error: failed to load CUDA bin file '%s'", cubin_file)
@@ -562,7 +586,7 @@ def get_CUDA_function(device_id, function_name):
         log(" loaded %s bytes", len(data))
         KERNELS[function_name] = data
     #now load from cubin:
-    start = monotonic_time()
+    start = monotonic()
     try:
         mod = driver.module_from_buffer(data)
     except Exception as e:
@@ -570,13 +594,13 @@ def get_CUDA_function(device_id, function_name):
         log.error("Error: failed to load module from buffer for '%s'", function_name)
         log.error(" %s", e)
         return None
-    log("get_CUDA_function(%s, %s) module=%s", device_id, function_name, mod)
+    log("get_CUDA_function(%s) module=%s", function_name, mod)
     try:
         fn = function_name
         CUDA_function = mod.get_function(fn)
     except driver.LogicError as e:
         raise Exception("failed to load '%s' from %s: %s" % (function_name, mod, e)) from None
-    end = monotonic_time()
+    end = monotonic()
     log("loading function %s from pre-compiled cubin took %.1fms", function_name, 1000.0*(end-start))
     return CUDA_function
 
