@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011-2021 Antoine Martin <antoine@xpra.org>
 # Copyright (C) 2008, 2009, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -11,6 +11,7 @@ import socket
 
 from xpra.net.common import ConnectionClosedException, IP_SOCKTYPES, TCP_SOCKTYPES
 from xpra.util import envint, envbool, csv
+from xpra.make_thread import start_thread
 from xpra.os_util import POSIX, LINUX, WIN32, OSX
 from xpra.platform.features import TCP_OPTIONS, IP_OPTIONS, SOCKET_OPTIONS
 from xpra.log import Logger
@@ -125,10 +126,10 @@ class Connection:
         self.timeout = 0
 
     def set_nodelay(self, nodelay : bool):
-        pass
+        """ TCP sockets override this method  """
 
     def set_cork(self, cork : bool):
-        pass
+        """ TCP sockets override this method  """
 
     def is_active(self) -> bool:
         return self.active
@@ -221,27 +222,36 @@ class TwoFileConnection(Connection):
     def close(self):
         log("%s.close() close callback=%s, readable=%s, writeable=%s",
             self, self._close_cb, self._readable, self._writeable)
-        Connection.close(self)
+        super().close()
         cc = self._close_cb
         if cc:
             self._close_cb = None
             log("%s.close() calling %s", self, cc)
-            cc()
-        try:
-            self._readable.close()
-        except IOError as e:
-            log("%s.close() %s", self._readable, e)
-        try:
-            self._writeable.close()
-        except IOError as e:
-            log("%s.close() %s", self._writeable, e)
+            try:
+                cc()
+            except Exception:
+                log.error("%s.close() error on callback %s", self, cc, exc_info=True)
+        def close_files_thread():
+            log("close_files_thread() _readable=%s", self._readable)
+            log("close_files_thread() calling %s", self._readable.close)
+            try:
+                self._readable.close()
+            except IOError as e:
+                log("close_files_thread() %s", self._readable, e)
+            log("close_files_thread() _writeable=%s", self._writeable)
+            log("close_files_thread() calling %s", self._writeable.close)
+            try:
+                self._writeable.close()
+            except IOError as e:
+                log("close_files_thread() %s", self._writeable, e)
+        start_thread(close_files_thread, "close-files-thread", daemon=True)
         log("%s.close() done", self)
 
     def __repr__(self):
         return "Pipe(%s)" % str(self.target)
 
     def get_info(self) -> dict:
-        d = Connection.get_info(self)
+        d = super().get_info()
         d.update({
             "type"  : "pipe",
             "pipe"  : {
@@ -328,6 +338,7 @@ class SocketConnection(Connection):
             log("changed %s socket to cork=%s", self.socktype, cork)
 
     def peek(self, n : int):
+        log("%s(%s, MSG_PEEK)", self._socket.recv, n)
         return self._socket.recv(n, socket.MSG_PEEK)
 
     def read(self, n : int):
@@ -343,8 +354,7 @@ class SocketConnection(Connection):
         except IOError:
             i = s
         log("%s.close() for socket=%s", self, i)
-        Connection.close(self)
-        #meaningless for udp:
+        super().close()
         try:
             s.settimeout(0)
         except IOError:
@@ -377,7 +387,7 @@ class SocketConnection(Connection):
         return "%s %s:%s" % (self.socktype, self.protocol_type, pretty_socket(self.local))
 
     def get_info(self) -> dict:
-        d = Connection.get_info(self)
+        d = super().get_info()
         try:
             d["remote"] = self.remote or ""
             d["protocol-type"] = self.protocol_type
@@ -418,7 +428,7 @@ class SocketConnection(Connection):
             if fd:
                 info["fileno"] = fd
             #ie: self.local = ("192.168.1.7", "14500")
-            log("do_get_socket_info(%s) fd=%s, local=%s", fd, self.local)
+            log("do_get_socket_info(%s) fd=%s, local=%s", s, fd, self.local)
             if self.local and len(self.local)==2:
                 from xpra.net.net_util import get_interface
                 iface = get_interface(self.local[0])
@@ -586,7 +596,7 @@ class SSLSocketConnection(PeekableSocketConnection):
         return super().can_retry(e)
 
     def get_info(self) -> dict:
-        i = SocketConnection.get_info(self)
+        i = super().get_info()
         i["ssl"] = True
         for k,fn in {
                      "compression"      : "compression",
@@ -613,9 +623,12 @@ class SSLSocketConnection(PeekableSocketConnection):
 
 def set_socket_timeout(conn, timeout=None):
     #FIXME: this is ugly, but less intrusive than the alternative?
-    log("set_socket_timeout(%s, %s)", conn, timeout)
     if isinstance(conn, SocketConnection):
+        sock = conn._socket
+        log("set_socket_timeout(%s, %s) applied to %s", conn, timeout, sock)
         conn._socket.settimeout(timeout)
+    else:
+        log("set_socket_timeout(%s, %s) ignored for %s", conn, timeout, type(conn))
 
 
 def log_new_connection(conn, socket_info=""):
@@ -634,17 +647,15 @@ def log_new_connection(conn, socket_info=""):
         sockname = ""
     log("log_new_connection(%s, %s) type=%s, sock=%s, sockname=%s, address=%s, peername=%s",
         conn, socket_info, type(conn), sock, sockname, address, peername)
+    log.info("New %s connection received", socktype)
     if peername:
         frominfo = pretty_socket(peername)
-        log.info("New %s connection received", socktype)
         log.info(" from '%s'", pretty_socket(frominfo))
         if socket_info:
             log.info(" on '%s'", pretty_socket(socket_info))
     elif socktype=="unix-domain":
         frominfo = sockname
-        log.info("New %s connection received", socktype)
         log.info(" on '%s'", frominfo)
     else:
-        log.info("New %s connection received")
         if socket_info:
             log.info(" on %s", pretty_socket(socket_info))
