@@ -7,18 +7,17 @@ import sys
 import os
 import shlex
 import socket
-import signal
-from time import sleep
+from time import sleep, monotonic
 from subprocess import PIPE, Popen
 
 from xpra.scripts.main import (
     InitException, InitExit,
     shellquote, host_target_string,
-    get_pinentry_command, run_pinentry_getpin, run_pinentry_confirm,
     )
 from xpra.platform.paths import get_ssh_known_hosts_files
-from xpra.platform import get_username
+from xpra.platform.info import get_username
 from xpra.scripts.config import parse_bool
+from xpra.scripts.pinentry_wrapper import input_pass, confirm
 from xpra.net.bytestreams import SocketConnection, SOCKET_TIMEOUT, ConnectionClosedException
 from xpra.make_thread import start_thread
 from xpra.exit_codes import (
@@ -26,21 +25,25 @@ from xpra.exit_codes import (
     EXIT_CONNECTION_FAILED,
     )
 from xpra.os_util import (
-    bytestostr, osexpand, load_binary_file, monotonic_time,
-    nomodule_context, umask_context, is_main_thread,
-    use_gui_prompt,
+    bytestostr, osexpand, load_binary_file,
+    nomodule_context, umask_context,
+    restore_script_env, get_saved_env,
     WIN32, OSX, POSIX,
     )
 from xpra.util import envint, envbool, envfloat, engs, csv
 from xpra.log import Logger, is_debug_enabled
 
+#pylint: disable=import-outside-toplevel
+
 log = Logger("network", "ssh")
+if log.is_debug_enabled():
+    import logging
+    logging.getLogger("paramiko").setLevel(logging.DEBUG)
+
 
 INITENV_COMMAND = os.environ.get("XPRA_INITENV_COMMAND", "")    #"xpra initenv"
 WINDOW_SIZE = envint("XPRA_SSH_WINDOW_SIZE", 2**27-1)
 TIMEOUT = envint("XPRA_SSH_TIMEOUT", 60)
-SKIP_UI = envbool("XPRA_SKIP_UI", False)
-PINENTRY = envbool("XPRA_SSH_PINENTRY", POSIX and not OSX)
 
 VERIFY_HOSTKEY = envbool("XPRA_SSH_VERIFY_HOSTKEY", True)
 VERIFY_STRICT = envbool("XPRA_SSH_VERIFY_STRICT", False)
@@ -50,7 +53,6 @@ NONE_AUTH = envbool("XPRA_SSH_NONE_AUTH", True)
 PASSWORD_AUTH = envbool("XPRA_SSH_PASSWORD_AUTH", True)
 AGENT_AUTH = envbool("XPRA_SSH_AGENT_AUTH", True)
 KEY_AUTH = envbool("XPRA_SSH_KEY_AUTH", True)
-PASSWORD_AUTH = envbool("XPRA_SSH_PASSWORD_AUTH", True)
 PASSWORD_RETRY = envint("XPRA_SSH_PASSWORD_RETRY", 2)
 assert PASSWORD_RETRY>=0
 MAGIC_QUOTES = envbool("XPRA_SSH_MAGIC_QUOTES", True)
@@ -69,101 +71,6 @@ def keymd5(k) -> str:
     return s
 
 
-def force_focus():
-    from xpra.platform.gui import force_focus as _force_focus
-    _force_focus()
-
-def dialog_run(dialog) -> int:
-    from gi.repository import GLib
-    if is_main_thread():
-        force_focus()
-        dialog.show()
-        try:
-            return dialog.run()
-        finally:
-            dialog.destroy()
-    #do a little dance if we're not running in the main thread:
-    #block this thread and wait for the main thread to run the dialog
-    from threading import Event
-    e = Event()
-    code = []
-    def main_thread_run():
-        force_focus()
-        dialog.show()
-        try:
-            r = dialog.run()
-        finally:
-            dialog.destroy()
-        code.append(r)
-        e.set()
-    GLib.idle_add(main_thread_run)
-    e.wait()
-    log("dialog_run(%s) code=%s", dialog, code)
-    return code[0]
-
-def dialog_pass(title="Password Input", prompt="enter password", icon="") -> str:
-    if PINENTRY:
-        pinentry_cmd = get_pinentry_command()
-        if pinentry_cmd:
-            return run_pinentry_getpin(pinentry_cmd, title, prompt)
-    from xpra.client.gtk_base.pass_dialog import PasswordInputDialogWindow
-    dialog = PasswordInputDialogWindow(title, prompt, icon)
-    try:
-        if dialog_run(dialog)==0:
-            return dialog.get_password()
-        return None
-    finally:
-        dialog.destroy()
-
-def dialog_confirm(title, prompt, qinfo=(), icon="", buttons=(("OK", 1),)) -> int:
-    from xpra.client.gtk_base.confirm_dialog import ConfirmDialogWindow
-    dialog = ConfirmDialogWindow(title, prompt, qinfo, icon, buttons)
-    try:
-        r = dialog_run(dialog)
-    finally:
-        dialog.destroy()
-    return r
-
-
-def confirm_key(info=(), title="Confirm Key", prompt="Are you sure you want to continue connecting?") -> bool:
-    if SKIP_UI:
-        return False
-    from xpra.platform.paths import get_icon_filename
-    if PINENTRY:
-        pinentry_cmd = get_pinentry_command()
-        if pinentry_cmd:
-            messages = list(info)+["", prompt]
-            return run_pinentry_confirm(pinentry_cmd, title, "%0A".join(messages))
-    if use_gui_prompt():
-        icon = get_icon_filename("authentication", "png") or ""
-        code = dialog_confirm(title, prompt, info, icon, buttons=[("yes", 200), ("NO", 201)])
-        log("dialog return code=%s", code)
-        r = code==200
-        log.info("host key %sconfirmed", ["not ", ""][r])
-        return r
-    log("confirm_key(%r) will use stdin prompt", info)
-    prompt = "Are you sure you want to continue connecting (yes/NO)? "
-    sys.stderr.write(os.linesep.join(info)+os.linesep+prompt)
-    try:
-        v = sys.stdin.readline().rstrip(os.linesep)
-    except KeyboardInterrupt:
-        sys.exit(128+signal.SIGINT)
-    return v and v.lower() in ("y", "yes")
-
-def input_pass(prompt) -> str:
-    if SKIP_UI:
-        return None
-    from xpra.platform.paths import get_icon_filename
-    if PINENTRY or use_gui_prompt():
-        icon = get_icon_filename("authentication", "png") or ""
-        return dialog_pass("Password Input", prompt, icon)
-    from getpass import getpass
-    try:
-        return getpass(prompt)
-    except KeyboardInterrupt:
-        sys.exit(128+signal.SIGINT)
-
-
 class SSHSocketConnection(SocketConnection):
 
     def __init__(self, ssh_channel, sock, sockname, peername, target, info=None, socket_options=None):
@@ -200,7 +107,7 @@ class SSHSocketConnection(SocketConnection):
         return self.do_get_socket_info(self._raw_socket)
 
     def get_info(self) -> dict:
-        i = SocketConnection.get_info(self)
+        i = super().get_info()
         s = self._socket
         if s:
             i["ssh-channel"] = {
@@ -248,7 +155,6 @@ class SSHProxyCommandConnection(SSHSocketConnection):
 
 def ssh_paramiko_connect_to(display_desc):
     #plain socket attributes:
-    dtype = display_desc["type"]
     host = display_desc["host"]
     port = display_desc.get("ssh-port", 22)
     #ssh and command attributes:
@@ -274,6 +180,10 @@ def ssh_paramiko_connect_to(display_desc):
             keyfiles.insert(0, keyfile)
         return keyfiles
 
+    def fail(msg):
+        log("ssh_paramiko_connect_to(%s)", display_desc, exc_info=True)
+        raise InitExit(EXIT_SSH_FAILURE, msg) from None
+
     with nogssapi_context():
         from paramiko import SSHConfig, ProxyCommand
         ssh_config = SSHConfig()
@@ -291,16 +201,18 @@ def ssh_paramiko_connect_to(display_desc):
             host_config = ssh_config.lookup(host)
             if host_config:
                 log("got host config for '%s': %s", host, host_config)
-                chost = host_config.get("hostname", host)
-                cusername = host_config.get("user", username)
-                cport = host_config.get("port", port)
-                try:
-                    port = int(cport)
-                except (TypeError, ValueError):
-                    raise InitExit(EXIT_SSH_FAILURE, "invalid ssh port specified: '%s'" % cport) from None
+                host = host_config.get("hostname", host)
+                if "username" not in display_desc:
+                    username = host_config.get("user", username)
+                if "ssh-port" not in display_desc:
+                    port = host_config.get("port", port)
+                    try:
+                        port = int(port)
+                    except (TypeError, ValueError):
+                        raise InitExit(EXIT_SSH_FAILURE, "invalid ssh port specified: '%s'" % port) from None
                 proxycommand = host_config.get("proxycommand")
                 if proxycommand:
-                    log("found proxycommand='%s' for host '%s'", proxycommand, chost)
+                    log("found proxycommand='%s' for host '%s'", proxycommand, host)
                     sock = ProxyCommand(proxycommand)
                     log("ProxyCommand(%s)=%s", proxycommand, sock)
                     from xpra.child_reaper import getChildReaper
@@ -314,18 +226,18 @@ def ssh_paramiko_connect_to(display_desc):
                     from paramiko.client import SSHClient
                     ssh_client = SSHClient()
                     ssh_client.load_system_host_keys()
-                    log("ssh proxy command connect to %s", (chost, cport, sock))
-                    ssh_client.connect(chost, cport, sock=sock)
+                    log("ssh proxy command connect to %s", (host, port, sock))
+                    ssh_client.connect(host, port, sock=sock)
                     transport = ssh_client.get_transport()
-                    do_ssh_paramiko_connect_to(transport, chost,
-                                               cusername, password,
+                    do_ssh_paramiko_connect_to(transport, host,
+                                               username, password,
                                                host_config or ssh_config.lookup("*"),
                                                proxy_keys,
                                                paramiko_config)
                     chan = paramiko_run_remote_xpra(transport, proxy_command, remote_xpra, socket_dir, display_as_args)
-                    peername = (chost, cport)
+                    peername = (host, port)
                     conn = SSHProxyCommandConnection(chan, peername, peername, socket_info)
-                    conn.target = host_target_string("ssh", cusername, chost, port, display)
+                    conn.target = host_target_string("ssh", username, host, port, display)
                     conn.timeout = SOCKET_TIMEOUT
                     conn.start_stderr_reader()
                     conn.process = (sock.process, "ssh", cmd)
@@ -335,7 +247,7 @@ def ssh_paramiko_connect_to(display_desc):
                     return conn
 
         keys = get_keyfiles(host_config)
-        from xpra.scripts.main import socket_connect
+        from xpra.net.socket_util import socket_connect
         from paramiko.transport import Transport
         from paramiko import SSHException
         if "proxy_host" in display_desc:
@@ -344,14 +256,13 @@ def ssh_paramiko_connect_to(display_desc):
             proxy_username = display_desc.get("proxy_username", username)
             proxy_password = display_desc.get("proxy_password", password)
             proxy_keys = get_keyfiles(host_config, "proxy_key")
-            sock = socket_connect(dtype, proxy_host, proxy_port)
+            sock = socket_connect(proxy_host, proxy_port)
             middle_transport = Transport(sock)
             middle_transport.use_compression(False)
             try:
                 middle_transport.start_client()
             except SSHException as e:
-                log("start_client()", exc_info=True)
-                raise InitExit(EXIT_SSH_FAILURE, "SSH negotiation failed: %s" % e) from None
+                fail("SSH proxy transport negotiation failed: %s" % e)
             proxy_host_config = ssh_config.lookup(host)
             do_ssh_paramiko_connect_to(middle_transport, proxy_host,
                                        proxy_username, proxy_password,
@@ -366,8 +277,7 @@ def ssh_paramiko_connect_to(display_desc):
             try:
                 transport.start_client()
             except SSHException as e:
-                log("start_client()", exc_info=True)
-                raise InitExit(EXIT_SSH_FAILURE, "SSH negotiation failed: %s" % e) from None
+                fail("SSH transport negotiation failed: %s" % e)
             do_ssh_paramiko_connect_to(transport, host,
                                        username, password,
                                        host_config or ssh_config.lookup("*"),
@@ -386,7 +296,7 @@ def ssh_paramiko_connect_to(display_desc):
 
         #plain TCP connection to the server,
         #we open it then give the socket to paramiko:
-        sock = socket_connect(dtype, host, port)
+        sock = socket_connect(host, port)
         sockname = sock.getsockname()
         peername = sock.getpeername()
         log("paramiko socket_connect: sockname=%s, peername=%s", sockname, peername)
@@ -395,13 +305,19 @@ def ssh_paramiko_connect_to(display_desc):
         try:
             transport.start_client()
         except SSHException as e:
-            log("start_client()", exc_info=True)
-            raise InitExit(EXIT_SSH_FAILURE, "SSH negotiation failed: %s" % e) from None
+            fail("SSH negotiation failed: %s" % e)
         do_ssh_paramiko_connect_to(transport, host, username, password,
                                    host_config or ssh_config.lookup("*"),
                                    keys,
                                    paramiko_config)
-        chan = paramiko_run_remote_xpra(transport, proxy_command, remote_xpra, socket_dir, display_as_args)
+        remote_port = display_desc.get("remote_port", 0)
+        if remote_port:
+            #we want to connect directly to a remote port,
+            #we don't need to run a command
+            chan = transport.open_channel("direct-tcpip", ("localhost", remote_port), ('localhost', 0))
+            log("direct channel to remote port %i : %s", remote_port, chan)
+        else:
+            chan = paramiko_run_remote_xpra(transport, proxy_command, remote_xpra, socket_dir, display_as_args)
         conn = SSHSocketConnection(chan, sock, sockname, peername, (host, port), socket_info)
         conn.target = host_target_string("ssh", username, host, port, display)
         conn.timeout = SOCKET_TIMEOUT
@@ -479,8 +395,8 @@ def do_ssh_paramiko_connect_to(transport, host, username, password, host_config=
                     dnscheck = check_host_key(host, host_key)
                 except ImportError as e:
                     log("verifyhostkeydns failed", exc_info=True)
-                    log.warn("Warning: cannot check SSHFP DNS records")
-                    log.warn(" %s", e)
+                    log.info("cannot check SSHFP DNS records")
+                    log.info(" %s", e)
             log("dnscheck=%s", dnscheck)
             def adddnscheckinfo(q):
                 if dnscheck is not True:
@@ -519,10 +435,10 @@ keymd5(host_key),
                     sys.stderr.write(os.linesep.join(qinfo))
                     transport.close()
                     raise InitExit(EXIT_SSH_KEY_FAILURE, "SSH Host key has changed")
-                if not confirm_key(qinfo):
+                if not confirm(qinfo):
                     transport.close()
                     raise InitExit(EXIT_SSH_KEY_FAILURE, "SSH Host key has changed")
-
+                log.info("host key confirmed")
             else:
                 assert (not keys) or (host_key.get_name() not in keys)
                 if not keys:
@@ -535,10 +451,10 @@ keymd5(host_key),
                     keymd5(host_key),
                     ]
                 adddnscheckinfo(qinfo)
-                if not confirm_key(qinfo):
+                if not confirm(qinfo):
                     transport.close()
                     raise InitExit(EXIT_SSH_KEY_FAILURE, "Unknown SSH host '%s'" % host)
-
+                log.info("host key confirmed")
             if configbool("addkey", ADD_KEY):
                 try:
                     if not host_keys_filename:
@@ -625,7 +541,8 @@ keymd5(host_key),
                         except SSHException as e:
                             log("from_private_key_file", exc_info=True)
                             log.info("cannot load key from file '%s':", keyfile_path)
-                            log.info(" %s", e)
+                            for emsg in str(e).split(". "):
+                                log.info(" %s.", emsg)
                     break
                 except Exception as e:
                     log("auth_publickey() loading as %s", pkey_classname, exc_info=True)
@@ -635,7 +552,12 @@ keymd5(host_key),
                         log.warn(" this file seems to be using OpenSSH's own format")
                         log.warn(" please convert it to something more standard (ie: PEM)")
                         log.warn(" so it can be used with the paramiko backend")
-                        log.warn(" or switch to the OpenSSH backend with '--ssh=ssh'")
+                        if WIN32:
+                            log.warn(" or switch to the Putty Plink backend with:")
+                            log.warn(" '--ssh=plink -ssh -agent'")
+                        else:
+                            log.warn(" or switch to the OpenSSH backend with:")
+                            log.warn(" '--ssh=ssh'")
             if key:
                 log("auth_publickey using %s as %s: %s", keyfile_path, pkey_classname, keymd5(key))
                 try:
@@ -664,14 +586,16 @@ keymd5(host_key),
         except SSHException as e:
             log("auth_password(..)", exc_info=True)
             log.info("SSH password authentication failed:")
-            log.info(" %s", getattr(e, "message", e))
+            for emsg in getattr(e, "message", str(e)).split(";"):
+                log.info(" %s", emsg)
 
     def auth_interactive():
         log("trying interactive authentication")
         class iauthhandler:
             def __init__(self):
                 self.authcount = 0
-            def handlestuff(self, _title, _instructions, prompt_list):
+            def handle_request(self, title, instructions, prompt_list):
+                log("handle_request%s counter=%i", (title, instructions, prompt_list), self.authcount)
                 p = []
                 for pent in prompt_list:
                     if self.authcount==0 and password:
@@ -679,14 +603,16 @@ keymd5(host_key),
                     else:
                         p.append(input_pass(pent[0]))
                     self.authcount += 1
+                log("handle_request(..) returning %i values", len(p))
                 return p
         try:
             myiauthhandler = iauthhandler()
-            transport.auth_interactive(username, myiauthhandler.handlestuff, "")
+            transport.auth_interactive(username, myiauthhandler.handle_request, "")
         except SSHException as e:
             log("auth_interactive(..)", exc_info=True)
             log.info("SSH password authentication failed:")
-            log.info(" %s", getattr(e, "message", e))
+            for emsg in getattr(e, "message", str(e)).split(";"):
+                log.info(" %s", emsg)
 
     banner = transport.get_banner()
     if banner:
@@ -758,9 +684,9 @@ def paramiko_run_test_command(transport, cmd):
         raise InitExit(EXIT_SSH_FAILURE, "failed to open SSH session: %s" % e) from None
     chan.exec_command(cmd)
     log("exec_command returned")
-    start = monotonic_time()
+    start = monotonic()
     while not chan.exit_status_ready():
-        if monotonic_time()-start>TEST_COMMAND_TIMEOUT:
+        if monotonic()-start>TEST_COMMAND_TIMEOUT:
             chan.close()
             raise InitException("SSH test command '%s' timed out" % cmd)
         log("exit status is not ready yet, sleeping")
@@ -808,16 +734,23 @@ def paramiko_run_remote_xpra(transport, xpra_proxy_command=None, remote_xpra=Non
             #assume this path exists
             pass
         else:
-            #assume Posix and find that command:
-            r = rtc("which %s" % xpra_cmd)
+            #assume POSIX and find that command:
+            r = rtc("command -v %s" % xpra_cmd)
             if r[2]!=0:
                 continue
-            if r[0]:
-                #use the actual path returned by 'which':
+            out = r[0]
+            if out:
+                #use the actual path returned by 'command -v':
+                if not isinstance(out, str):
+                    out = bytestostr(out)
                 try:
-                    xpra_cmd = r[0].decode().splitlines()[-1].rstrip("\n\r ").lstrip("\t ")
-                except Exception:
-                    pass
+                    xpra_cmd = out.splitlines()[-1].rstrip("\n\r ").lstrip("\t ")
+                except Exception as e:
+                    log("cannot get command from %r: %s", xpra_cmd, e)
+                else:
+                    if xpra_cmd.startswith("alias %s=" % xpra_cmd):
+                        #ie: "alias xpra='xpra -d proxy'" -> "xpra -d proxy"
+                        xpra_cmd = xpra_cmd.split("=", 1)[1].strip("'")
         log("adding xpra_cmd='%s'", xpra_cmd)
         if xpra_cmd in tried:
             continue
@@ -862,6 +795,12 @@ def ssh_exec_connect_to(display_desc, opts=None, debug_cb=None, ssh_fail_cb=ssh_
         cmd = list(display_desc["full_ssh"])
         kwargs = {}
         env = display_desc.get("env")
+        if env is None:
+            env = get_saved_env()
+        if display_desc.get("is_putty"):
+            #special env used by plink:
+            env = os.environ.copy()
+            env["PLINK_PROTOCOL"] = "ssh"
         kwargs["stderr"] = sys.stderr
         if WIN32:
             from subprocess import CREATE_NEW_PROCESS_GROUP, CREATE_NEW_CONSOLE, STARTUPINFO, STARTF_USESHOWWINDOW
@@ -888,8 +827,8 @@ def ssh_exec_connect_to(display_desc, opts=None, debug_cb=None, ssh_fail_cb=ssh_
             else:
                 check = "elif"
             if x=="xpra":
-                #no absolute path, so use "which" to check that the command exists:
-                pc = ['%s which "%s" > /dev/null 2>&1; then' % (check, x)]
+                #no absolute path, so use "command -v" to check that the command exists:
+                pc = ['%s command -v "%s" > /dev/null 2>&1; then' % (check, x)]
             else:
                 pc = ['%s [ -x %s ]; then' % (check, x)]
             pc += [x] + proxy_command + [shellquote(x) for x in display_as_args]
@@ -923,8 +862,6 @@ def ssh_exec_connect_to(display_desc, opts=None, debug_cb=None, ssh_fail_cb=ssh_
                 #sshpass -e ssh ...
                 cmd.insert(0, sshpass_command)
                 cmd.insert(1, "-e")
-                if env is None:
-                    env = os.environ.copy()
                 env["SSHPASS"] = password
                 #the password will be used by ssh via sshpass,
                 #don't try to authenticate again over the ssh-proxy connection,
@@ -932,14 +869,16 @@ def ssh_exec_connect_to(display_desc, opts=None, debug_cb=None, ssh_fail_cb=ssh_
                 #authentication over unix-domain-sockets:
                 opts.password = None
                 del display_desc["password"]
-        if env:
-            kwargs["env"] = env
+
+        kwargs["env"] = restore_script_env(env)
+
         if is_debug_enabled("ssh"):
             log.info("executing ssh command: %s" % (" ".join("\"%s\"" % x for x in cmd)))
         child = Popen(cmd, stdin=PIPE, stdout=PIPE, **kwargs)
     except OSError as e:
+        cmd_info = " ".join("%r" % x for x in cmd)
         raise InitExit(EXIT_SSH_FAILURE,
-                       "Error running ssh command '%s': %s" % (" ".join("\"%s\"" % x for x in cmd), e))
+                       "Error running ssh command '%s': %s" % (cmd_info, e)) from None
     def abort_test(action):
         """ if ssh dies, we don't need to try to read/write from its sockets """
         e = child.poll()
