@@ -151,7 +151,7 @@ class OSX_Notifier(NotifierBase):
             self.gtk_notifier.close_notify(nid)
 
     def cleanup(self):
-        NotifierBase.cleanup(self)
+        super().cleanup()
         GLib.idle_add(self.notification_center.removeAllDeliveredNotifications)
 
 
@@ -419,7 +419,7 @@ def get_info():
     i = get_info_base()
     try:
         i["displays"] = get_displays_info()
-    except:
+    except Exception:
         log.error("Error: OSX get_display_info failed", exc_info=True)
     return i
 
@@ -427,12 +427,6 @@ def get_info():
 #keep track of the window object for each view
 VIEW_TO_WINDOW = weakref.WeakValueDictionary()
 
-def add_window_hooks(window):
-    pass
-
-def remove_window_hooks(window):
-    pass
-
 
 def get_CG_imagewrapper(rect=None):
     from xpra.codecs.image_wrapper import ImageWrapper
@@ -583,7 +577,8 @@ class Delegate(NSObject):
         if dx!=0 or dy!=0:
             client = window._client
             wid = window._id
-            client.wheel_event(wid, dx, dy)
+            pointer = window.get_mouse_position()
+            client.wheel_event(wid, dx, dy, pointer)
         return True
 
     @objc.python_method
@@ -623,7 +618,7 @@ class Delegate(NSObject):
                 self.delegate_cb("wake_callback")
             else:
                 self.delegate_cb("sleep_callback")
-        except:
+        except Exception:
             workspacelog.error("Error querying workspace info", exc_info=True)
 
     #def application_openFile_(self, application, fileName):
@@ -645,7 +640,7 @@ class Delegate(NSObject):
         if callback:
             try:
                 callback()
-            except:
+            except Exception:
                 log.error("Error in %s callback %s", name, callback, exc_info=True)
 
 
