@@ -35,15 +35,15 @@ ALPHA = {
 BTYPES = tuple((str, bytes, memoryview, bytearray))
 
 #ensure that picture_encode can deal with pixels as NSCFData:
-def patch_picture_encode():
+def patch_pixels_to_bytes():
     from CoreFoundation import CFDataGetBytes, CFDataGetLength  #@UnresolvedImport
     def pixels_to_bytes(v):
         if isinstance(v, BTYPES):
             return memoryview_to_bytes(v)
         l = CFDataGetLength(v)
         return CFDataGetBytes(v, (0, l), None)
-    from xpra.server import picture_encode
-    picture_encode.pixels_to_bytes = pixels_to_bytes
+    from xpra.codecs import rgb_transform
+    rgb_transform.pixels_to_bytes = pixels_to_bytes
 
 
 class OSXRootCapture:
@@ -55,7 +55,7 @@ class OSXRootCapture:
         return True
 
     def clean(self):
-        pass
+        """ nothing specific to do here on MacOS """
 
     def get_image(self, x, y, width, height):
         rect = (x, y, width, height)
@@ -81,11 +81,11 @@ class ShadowServer(GTKShadowServerBase):
         if image is None:
             log("cannot grab test screenshot - maybe you need to run this command whilst logged in via the UI")
             raise InitExit(EXIT_FAILURE, "cannot grab pixels from the screen, make sure this command is launched from a GUI session")
-        patch_picture_encode()
+        patch_pixels_to_bytes()
         self.refresh_count = 0
         self.refresh_rectangle_count = 0
         self.refresh_registered = False
-        GTKShadowServerBase.__init__(self)
+        super().__init__()
 
     def init(self, opts):
         super().init(opts)
@@ -166,10 +166,11 @@ class ShadowServer(GTKShadowServerBase):
 
 
     def do_process_mouse_common(self, proto, wid, pointer, *args):
-        assert proto in self._server_sources
+        if proto not in self._server_sources:
+            return False
         assert wid in self._id_to_window
         CG.CGWarpMouseCursorPosition(pointer[:2])
-        return pointer
+        return True
 
     def fake_key(self, keycode, press):
         e = CG.CGEventCreateKeyboardEvent(None, keycode, press)
@@ -186,30 +187,30 @@ class ShadowServer(GTKShadowServerBase):
         if pointer:
             self.button_action(pointer, button, pressed, -1, *args)
 
-    def button_action(self, pointer, button, pressed, _deviceid=-1, *args):
+    def button_action(self, pointer, button, pressed, _deviceid=-1, *_args):
         if button<=3:
             #we should be using CGEventCreateMouseEvent
             #instead we clear previous clicks when a "higher" button is pressed... oh well
-            args = [pointer[:2], 1, button]
+            event = [pointer[:2], 1, button]
             for i in range(button):
-                args.append(i==(button-1) and pressed)
-            r = CG.CGPostMouseEvent(*args)
-            log("CG.CGPostMouseEvent%s=%s", args, r)
-        else:
-            if not pressed:
-                #we don't simulate press/unpress
-                #so just ignore unpressed events
-                return
-            wheel = (button-2)//2
-            direction = 1-(((button-2) % 2)*2)
-            args = [wheel]
-            for i in range(wheel):
-                if i!=(wheel-1):
-                    args.append(0)
-                else:
-                    args.append(direction)
-            r = CG.CGPostScrollWheelEvent(*args)
-            log("CG.CGPostScrollWheelEvent%s=%s", args, r)
+                event.append(i==(button-1) and pressed)
+            r = CG.CGPostMouseEvent(*event)
+            log("CG.CGPostMouseEvent%s=%s", event, r)
+            return
+        if not pressed:
+            #we don't simulate press/unpress
+            #so just ignore unpressed events
+            return
+        wheel = (button-2)//2
+        direction = 1-(((button-2) % 2)*2)
+        event = [wheel]
+        for i in range(wheel):
+            if i!=(wheel-1):
+                event.append(0)
+            else:
+                event.append(direction)
+        r = CG.CGPostScrollWheelEvent(*event)
+        log("CG.CGPostScrollWheelEvent%s=%s", event, r)
 
     def make_hello(self, source):
         capabilities = GTKServerBase.make_hello(self, source)
