@@ -1,30 +1,38 @@
 # This file is part of Xpra.
-# Copyright (C) 2017-2020 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2017-2021 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import sys
+import shlex
 import os.path
 
 from xpra.util import envbool
 from xpra.os_util import (
     OSX, POSIX,
     which,
-    shellsub, getuid,
+    shellsub,
     get_util_logger,
     osexpand, umask_context,
+    close_all_fds,
     )
 from xpra.platform.dotxpra import norm_makepath
 from xpra.scripts.config import InitException
 
+UINPUT_UUID_LEN = 12
+
 
 def source_env(source=()) -> dict:
     log = get_util_logger()
+    log("source_env(%s)", source)
     env = {}
     for f in source:
+        if not f:
+            continue
         e = env_from_sourcing(f)
         log("source_env %s=%s", f, e)
         env.update(e)
+    log("source_env(%s)=%s", source, env)
     return env
 
 
@@ -35,14 +43,15 @@ def env_from_sourcing(file_to_source_path, include_unexported_variables=False):
     import subprocess
     from xpra.log import Logger
     log = Logger("exec")
-    filename = file_to_source_path
-    if not os.path.exists(file_to_source_path):
-        filename = which(file_to_source_path)
-        if not filename:
-            log.error("Error: cannot find file '%s' to source", file_to_source_path)
-            return {}
+    cmd = shlex.split(file_to_source_path)
+    filename = which(cmd[0])
+    if not filename:
+        log.error("Error: cannot find command '%s' to execute", cmd[0])
+        log.error(" for sourcing '%s'", file_to_source_path)
+        return {}
     if not os.path.isabs(filename):
         filename = os.path.abspath(filename)
+    cmd[0] = filename
     #figure out if this is a script to source,
     #or if we're meant to execute it directly
     try:
@@ -52,8 +61,9 @@ def env_from_sourcing(file_to_source_path, include_unexported_variables=False):
         log.error("Error: failed to read from '%s'", filename)
         log.error(" %s", e)
         first_line = b""
+    else:
+        log("first line of '%s': %r", filename, first_line)
     if first_line.startswith(b"\x7fELF") or b"\x00" in first_line:
-        cmd = [filename]
         def decode(out):
             env = {}
             for line in out.splitlines():
@@ -64,7 +74,7 @@ def env_from_sourcing(file_to_source_path, include_unexported_variables=False):
             return env
     else:
         source = '%ssource %s' % ("set -a && " if include_unexported_variables else "", filename)
-        dump = '/usr/bin/python -c "import os, json;print(json.dumps(dict(os.environ)))"'
+        dump = 'python%i.%i -c "import os, json;print(json.dumps(dict(os.environ)))"' % (sys.version_info.major, sys.version_info.minor)
         cmd = ['/bin/bash', '-c', '%s && %s' % (source, dump)]
         def decode(out):
             try:
@@ -101,10 +111,9 @@ def env_from_sourcing(file_to_source_path, include_unexported_variables=False):
 def sh_quotemeta(s):
     return b"'" + s.replace(b"'", b"'\\''") + b"'"
 
-def xpra_runner_shell_script(xpra_file, starting_dir, socket_dir):
-    script = []
-    script.append(b"#!/bin/sh\n")
-    for var, value in os.environb.items():
+def xpra_env_shell_script(socket_dir, env):
+    script = [b"#!/bin/sh", b""]
+    for var, value in env.items():
         # these aren't used by xpra, and some should not be exposed
         # as they are either irrelevant or simply do not match
         # the new environment used by xpra
@@ -127,20 +136,26 @@ def xpra_runner_shell_script(xpra_file, starting_dir, socket_dir):
             pval = value.split(pathsep)      #ie: ["/usr/bin", "/usr/local/bin", "/usr/bin"]
             seen = set()
             value = pathsep.join(x for x in pval if not (x in seen or seen.add(x)))
-            script.append(b"%s=%s:\"$%s\"; export %s\n"
+            script.append(b"%s=%s:\"$%s\"; export %s"
                           % (var, sh_quotemeta(value), var, var))
         else:
-            script.append(b"%s=%s; export %s\n"
+            script.append(b"%s=%s; export %s"
                           % (var, sh_quotemeta(value), var))
     #XPRA_SOCKET_DIR is a special case, we want to honour it
     #when it is specified, but the client may override it:
     if socket_dir:
-        script.append(b'if [ -z "${XPRA_SOCKET_DIR}" ]; then\n')
-        script.append(b'    XPRA_SOCKET_DIR="%s"; export XPRA_SOCKET_DIR\n' % sh_quotemeta(os.path.expanduser(socket_dir).encode()))
-        script.append(b'fi\n')
+        script.append(b'if [ -z "${XPRA_SOCKET_DIR}" ]; then')
+        script.append(b'    XPRA_SOCKET_DIR="%s"; export XPRA_SOCKET_DIR' %
+                      sh_quotemeta(os.path.expanduser(socket_dir).encode()))
+        script.append(b'fi')
+    script.append(b"")
+    return b"\n".join(script)
+
+def xpra_runner_shell_script(xpra_file, starting_dir):
+    script = []
     # We ignore failures in cd'ing, b/c it's entirely possible that we were
     # started from some temporary directory and all paths are absolute.
-    script.append(b"cd %s\n" % sh_quotemeta(starting_dir.encode()))
+    script.append(b"cd %s" % sh_quotemeta(starting_dir.encode()))
     if OSX:
         #OSX contortions:
         #The executable is the python interpreter,
@@ -151,7 +166,7 @@ def xpra_runner_shell_script(xpra_file, starting_dir, socket_dir):
             sexec = os.path.join(sexec[:bini], "Resources", "MacOS", "Xpra")
         script.append(b"_XPRA_SCRIPT=%s\n" % (sh_quotemeta(sexec.encode()),))
         script.append(b"""
-if which "$_XPRA_SCRIPT" > /dev/null; then
+if command -v "$_XPRA_SCRIPT" > /dev/null; then
     # Happypath:
     exec "$_XPRA_SCRIPT" "$@"
 else
@@ -160,10 +175,10 @@ else
 fi
 """)
     else:
-        script.append(b"_XPRA_PYTHON=%s\n" % (sh_quotemeta(sys.executable.encode()),))
-        script.append(b"_XPRA_SCRIPT=%s\n" % (sh_quotemeta(xpra_file.encode()),))
+        script.append(b"_XPRA_PYTHON=%s" % (sh_quotemeta(sys.executable.encode()),))
+        script.append(b"_XPRA_SCRIPT=%s" % (sh_quotemeta(xpra_file.encode()),))
         script.append(b"""
-if which "$_XPRA_PYTHON" > /dev/null && [ -e "$_XPRA_SCRIPT" ]; then
+if command -v "$_XPRA_PYTHON" > /dev/null && [ -e "$_XPRA_SCRIPT" ]; then
     # Happypath:
     exec "$_XPRA_PYTHON" "$_XPRA_SCRIPT" "$@"
 else
@@ -175,7 +190,7 @@ END
     exec xpra "$@"
 fi
 """)
-    return b"".join(script)
+    return b"\n".join(script)
 
 def write_runner_shell_scripts(contents, overwrite=True):
     assert POSIX
@@ -220,26 +235,6 @@ def write_runner_shell_scripts(contents, overwrite=True):
             log.error(" %s\n", e)
 
 
-def find_log_dir(username="", uid=0, gid=0):
-    from xpra.platform.paths import get_default_log_dirs
-    errs  = []
-    for x in get_default_log_dirs():
-        v = osexpand(x, username, uid, gid)
-        if not os.path.exists(v):
-            if getuid()==0 and uid!=0:
-                continue
-            try:
-                os.mkdir(v, 0o700)
-            except Exception as e:
-                errs.append((v, e))
-                continue
-        return v
-    for d, e in errs:
-        sys.stderr.write("Error: cannot create log directory '%s':" % d)
-        sys.stderr.write(" %s\n" % e)
-    return None
-
-
 def open_log_file(logpath):
     """ renames the existing log file if it exists,
         then opens it for writing.
@@ -274,7 +269,6 @@ def select_log_file(log_dir, log_file, display_name):
     return logpath
 
 def redirect_std_to_log(logfd, *noclose_fds):
-    from xpra.os_util import close_all_fds
     # save current stdout/stderr to be able to print info
     # before exiting the non-deamon process
     # and closing those file descriptors definitively
@@ -313,7 +307,8 @@ def write_pidfile(pidfile):
     inode = 0
     try:
         with open(pidfile, "w") as f:
-            os.fchmod(f.fileno(), 0o600)
+            if POSIX:
+                os.fchmod(f.fileno(), 0o640)
             f.write("%s\n" % pidstr)
             try:
                 inode = os.fstat(f.fileno()).st_ino
@@ -380,6 +375,8 @@ def get_uinput_device_path(device):
     return None
 
 def has_uinput():
+    if not envbool("XPRA_UINPUT", True):
+        return False
     try:
         import uinput
         assert uinput
@@ -392,8 +389,7 @@ def has_uinput():
     except ImportError as e:
         log = get_util_logger()
         log("has_uinput()", exc_info=True)
-        log.info("cannot access python uinput module:")
-        log.info(" %s", e)
+        log.info("no uinput module")
         return False
     try:
         uinput.fdopen()         #@UndefinedVariable
@@ -407,7 +403,7 @@ def has_uinput():
 
 def create_uinput_device(uuid, uid, events, name):
     log = get_util_logger()
-    import uinput
+    import uinput  # @UnresolvedImport
     BUS_USB = 0x03
     #BUS_VIRTUAL = 0x06
     VENDOR = 0xffff
@@ -425,8 +421,6 @@ def create_uinput_device(uuid, uid, events, name):
             log.error(" make sure that the kernel module is loaded")
             log.error(" and that the /dev/uinput device exists:")
             log.error(" %s", e)
-        else:
-            log.info("cannot access uinput: %s", e)
         return None
     dev_path = get_uinput_device_path(device)
     if not dev_path:
@@ -472,7 +466,7 @@ def create_uinput_touchpad_device(uuid, uid):
 def create_uinput_devices(uinput_uuid, uid):
     log = get_util_logger()
     try:
-        import uinput
+        import uinput  # @UnresolvedImport
         assert uinput
     except (ImportError, NameError) as e:
         log.error("Error: cannot access python uinput module:")
