@@ -1,12 +1,13 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 # This file is part of Xpra.
 # Copyright (C) 2012-2020 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+from time import monotonic
 from xpra.x11.x11_server_core import X11ServerCore
-from xpra.os_util import monotonic_time, is_Wayland, get_loaded_kernel_modules
+from xpra.os_util import is_Wayland, get_loaded_kernel_modules
 from xpra.util import (
     envbool, envint, merge_dicts,
     XPRA_DISPLAY_NOTIFICATION_ID, XPRA_SHADOWWAYLAND_NOTIFICATION_ID,
@@ -38,6 +39,7 @@ if USE_NVFBC:
 
 
 class XImageCapture:
+    __slots__ = ("xshm", "xwindow")
     def __init__(self, xwindow):
         self.xshm = None
         self.xwindow = xwindow
@@ -87,7 +89,7 @@ class XImageCapture:
             log("no xshm, cannot get image")
             return None
         try:
-            start = monotonic_time()
+            start = monotonic()
             with xsync:
                 log("X11 shadow get_image, xshm=%s", self.xshm)
                 image = self.xshm.get_image(self.xwindow, x, y, width, height)
@@ -96,7 +98,7 @@ class XImageCapture:
             self._err(e)
             return None
         finally:
-            end = monotonic_time()
+            end = monotonic()
             log("X11 shadow captured %s pixels at %i MPixels/s using %s",
                 width*height, (width*height/(end-start))//1024//1024, ["GTK", "XSHM"][USE_XSHM])
 
@@ -202,7 +204,7 @@ class ShadowX11Server(GTKShadowServerBase, X11ServerCore):
                 if get_loaded_kernel_modules("vboxguest", "vboxvideo"):
                     body += "\nthis may be caused by the VirtualBox video driver."
                 title = "Shadow Capture Failure"
-            log("verify_capture: title=%r, body=%r", ss, title, body)
+            log("verify_capture: title=%r, body=%r", title, body)
             if title and body:
                 ss.may_notify(nid, title, body, icon_name="server")
         except Exception as e:
